@@ -19,6 +19,7 @@
  * @author Christian Grothoff
  */
 #include "platform.h"
+#include "microhttpd.h"
 #include <gnunet/gnunet_util_lib.h>
 #include <jansson.h>
 #include <pthread.h>
@@ -106,6 +107,15 @@ struct Exchange
    * hard limit?
    */
   bool limited;
+
+  /**
+   * Are we force-retrying a /keys download because some keys
+   * were missing (and we thus should not cherry-pick, as
+   * a major reason for a force-reload would be an
+   * exchange that has lost keys and backfilled them, which
+   * breaks keys downloads with cherry-picking).
+   */
+  bool force_retry;
 };
 
 
@@ -277,10 +287,12 @@ add_restriction (json_t *restrictions,
  * depending on the return value.
  *
  * @param keys information to persist
+ * @param first_retry earliest we may retry fetching the keys
  * @return transaction status
  */
 static enum GNUNET_DB_QueryStatus
-insert_keys_data (const struct TALER_EXCHANGE_Keys *keys)
+insert_keys_data (const struct TALER_EXCHANGE_Keys *keys,
+                  struct GNUNET_TIME_Absolute first_retry)
 {
   enum GNUNET_DB_QueryStatus qs;
 
@@ -307,7 +319,8 @@ insert_keys_data (const struct TALER_EXCHANGE_Keys *keys)
   }
 
   qs = db_plugin->insert_exchange_keys (db_plugin->cls,
-                                        keys);
+                                        keys,
+                                        first_retry);
   if (0 > qs)
   {
     GNUNET_break (GNUNET_DB_STATUS_SOFT_ERROR == qs);
@@ -408,12 +421,22 @@ insert_keys_data (const struct TALER_EXCHANGE_Keys *keys)
     } /* all fees for this method */
   } /* for all methods (i) */
 
+  GNUNET_log (GNUNET_ERROR_TYPE_INFO,
+              "Updated keys for %s, inserted %d signing keys, %d denom keys, %d fees-by-wire\n",
+              keys->exchange_url,
+              keys->num_sign_keys,
+              keys->num_denom_keys,
+              keys->fees_len);
+
   {
     struct GNUNET_DB_EventHeaderP es = {
       .size = ntohs (sizeof (es)),
       .type = ntohs (TALER_DBEVENT_MERCHANT_EXCHANGE_KEYS)
     };
 
+    GNUNET_log (GNUNET_ERROR_TYPE_INFO,
+                "Informing other processes about keys change for %s\n",
+                keys->exchange_url);
     db_plugin->event_notify (db_plugin->cls,
                              &es,
                              keys->exchange_url,
@@ -429,10 +452,12 @@ insert_keys_data (const struct TALER_EXCHANGE_Keys *keys)
  * that may care about them).
  *
  * @param keys the keys to store
+ * @param first_retry earliest we may retry fetching the keys
  * @return true on success
  */
 static bool
-store_keys (struct TALER_EXCHANGE_Keys *keys)
+store_keys (struct TALER_EXCHANGE_Keys *keys,
+            struct GNUNET_TIME_Absolute first_retry)
 {
   enum GNUNET_DB_QueryStatus qs;
 
@@ -448,8 +473,9 @@ store_keys (struct TALER_EXCHANGE_Keys *keys)
       return false;
     }
 
-    qs = insert_keys_data (keys);
-    if (qs < 0)
+    qs = insert_keys_data (keys,
+                           first_retry);
+    if (0 > qs)
     {
       db_plugin->rollback (db_plugin->cls);
       if (GNUNET_DB_STATUS_SOFT_ERROR == qs)
@@ -459,7 +485,7 @@ store_keys (struct TALER_EXCHANGE_Keys *keys)
     }
 
     qs = db_plugin->commit (db_plugin->cls);
-    if (qs < 0)
+    if (0 > qs)
     {
       db_plugin->rollback (db_plugin->cls);
       if (GNUNET_DB_STATUS_SOFT_ERROR == qs)
@@ -467,6 +493,7 @@ store_keys (struct TALER_EXCHANGE_Keys *keys)
       GNUNET_break (0);
       return false;
     }
+    break;
   } /* end of retry loop */
   if (qs < 0)
   {
@@ -493,6 +520,7 @@ cert_cb (
 {
   struct Exchange *e = cls;
   struct GNUNET_TIME_Absolute n;
+  struct GNUNET_TIME_Absolute first_retry;
 
   e->conn = NULL;
   switch (kr->hr.http_status)
@@ -508,6 +536,7 @@ cert_cb (
                   e->exchange_url,
                   keys->currency,
                   e->currency);
+      TALER_EXCHANGE_keys_decref (keys);
       break;
     }
     if (0 != GNUNET_memcmp (&keys->master_pub,
@@ -516,17 +545,26 @@ cert_cb (
       GNUNET_log (GNUNET_ERROR_TYPE_WARNING,
                   "Master public key in %skeys response does not match. Ignoring response.\n",
                   e->exchange_url);
+      TALER_EXCHANGE_keys_decref (keys);
       break;
     }
-    if (! store_keys (keys))
+    GNUNET_log (GNUNET_ERROR_TYPE_INFO,
+                "Got new keys for %s, updating database\n",
+                e->exchange_url);
+    first_retry = GNUNET_TIME_relative_to_absolute (
+      EXCHANGE_MAXFREQ);
+    if (! store_keys (keys,
+                      first_retry))
+    {
+      GNUNET_break (0);
+      TALER_EXCHANGE_keys_decref (keys);
       break;
-    e->keys = TALER_EXCHANGE_keys_incref (keys);
+    }
+    e->keys = keys;
     /* Reset back-off */
     e->retry_delay = EXCHANGE_MAXFREQ;
     /* limit retry */
-    e->first_retry
-      = GNUNET_TIME_relative_to_absolute (
-          EXCHANGE_MAXFREQ);
+    e->first_retry = first_retry;
     /* Limit by expiration */
     n = GNUNET_TIME_absolute_max (e->first_retry,
                                   keys->key_data_expiration.abs_time);
@@ -538,6 +576,7 @@ cert_cb (
     end_inquiry ();
     return;
   default:
+    GNUNET_break (NULL == keys);
     break;
   }
   /* Try again (soon-ish) */
@@ -579,11 +618,18 @@ download_keys (void *cls)
   }
   e->retry_delay
     = GNUNET_TIME_STD_BACKOFF (e->retry_delay);
+  GNUNET_log (GNUNET_ERROR_TYPE_INFO,
+              "Downloading keys from %s (%s)\n",
+              e->exchange_url,
+              e->force_retry ? "forced" : "regular");
   e->conn = TALER_EXCHANGE_get_keys (ctx,
                                      e->exchange_url,
-                                     e->keys,
+                                     e->force_retry
+                                     ? NULL
+                                     : e->keys,
                                      &cert_cb,
                                      e);
+  e->force_retry = false;
   if (NULL != e->conn)
   {
     active_inquiries++;
@@ -676,6 +722,7 @@ force_exchange_keys (void *cls,
                 true));
   if (NULL != e->retry_task)
     GNUNET_SCHEDULER_cancel (e->retry_task);
+  e->force_retry = true;
   e->retry_task
     = GNUNET_SCHEDULER_add_at (e->first_retry,
                                &download_keys,
@@ -801,6 +848,7 @@ accept_exchanges (void *cls,
 
       qs = db_plugin->select_exchange_keys (db_plugin->cls,
                                             url,
+                                            &e->first_retry,
                                             &keys);
       if (qs < 0)
       {
