@@ -43,7 +43,7 @@
  * #MERCHANT_PROTOCOL_CURRENT and #MERCHANT_PROTOCOL_AGE in
  * merchant_api_get_config.c!
  */
-#define MERCHANT_PROTOCOL_VERSION "23:1:11"
+#define MERCHANT_PROTOCOL_VERSION "24:0:12"
 
 
 /**
@@ -122,6 +122,12 @@ MH_handler_config (const struct TMH_RequestHandler *rh,
                                TMH_currency),
       GNUNET_JSON_pack_string ("payment_target_types",
                                TMH_allowed_payment_targets),
+      GNUNET_JSON_pack_time_rel ("default_refund_delay",
+                                 TMH_default_refund_delay),
+      GNUNET_JSON_pack_time_rel ("default_wire_transfer_delay",
+                                 TMH_default_wire_transfer_delay),
+      GNUNET_JSON_pack_time_rel ("default_pay_delay",
+                                 TMH_default_pay_delay),
       GNUNET_JSON_pack_string ("default_persona",
                                TMH_default_persona),
       GNUNET_JSON_pack_allow_null (
