@@ -941,8 +941,9 @@ reload_exchange_keys (struct TMH_Exchange *exchange)
     return;
   }
   GNUNET_log (GNUNET_ERROR_TYPE_INFO,
-              "Loaded /keys from database with %u accounts\n",
-              keys->accounts_len);
+              "Loaded /keys from database with %u accounts, %u fees\n",
+              keys->accounts_len,
+              keys->fees_len);
   if (GNUNET_OK !=
       process_wire_fees (exchange,
                          &keys->master_pub,
