@@ -21,13 +21,372 @@
 #include "platform.h"
 #include <gnunet/gnunet_util_lib.h>
 #include <gnunet/gnunet_db_lib.h>
-#include <taler/taler_util.h>
 #include <taler/taler_json_lib.h>
 #include "taler-merchant-httpd_helper.h"
 #include <taler/taler_templating_lib.h>
 #include <taler/taler_dbevents.h>
 
 
+enum GNUNET_GenericReturnValue
+TMH_parse_fractional_string (const char *value,
+                             int64_t *integer_part,
+                             uint32_t *fractional_part)
+{
+  const char *ptr;
+  uint64_t integer = 0;
+  uint32_t frac = 0;
+  unsigned int digits = 0;
+
+  GNUNET_assert (NULL != integer_part);
+  GNUNET_assert (NULL != fractional_part);
+
+  if (NULL == value)
+  {
+    GNUNET_break_op (0);
+    return GNUNET_SYSERR;
+  }
+  ptr = value;
+  if ('\0' == *ptr)
+  {
+    GNUNET_break_op (0);
+    return GNUNET_SYSERR;
+  }
+  if ('-' == *ptr)
+  {
+    GNUNET_break_op (0);
+    return GNUNET_SYSERR;
+  }
+  if (! isdigit ((unsigned char) *ptr))
+  {
+    GNUNET_break_op (0);
+    return GNUNET_SYSERR;
+  }
+  while (isdigit ((unsigned char) *ptr))
+  {
+    unsigned int digit = (unsigned int) (*ptr - '0');
+
+    if (integer > (UINT64_MAX - digit) / 10)
+    {
+      GNUNET_break_op (0);
+      return GNUNET_SYSERR;
+    }
+    integer = integer * 10 + digit;
+    ptr++;
+  }
+  if ('.' == *ptr)
+  {
+    ptr++;
+    if ('\0' == *ptr)
+    {
+      GNUNET_break_op (0);
+      return GNUNET_SYSERR;
+    }
+    while (isdigit ((unsigned char) *ptr))
+    {
+      unsigned int digit = (unsigned int) (*ptr - '0');
+
+      if (digits >= MERCHANT_UNIT_FRAC_MAX_DIGITS)
+      {
+        GNUNET_break_op (0);
+        return GNUNET_SYSERR;
+      }
+      frac = (uint32_t) (frac * 10 + digit);
+      digits++;
+      ptr++;
+    }
+    while (digits < MERCHANT_UNIT_FRAC_MAX_DIGITS)
+    {
+      frac *= 10;
+      digits++;
+    }
+  }
+  if ('\0' != *ptr)
+  {
+    GNUNET_break_op (0);
+    return GNUNET_SYSERR;
+  }
+  if (integer > (uint64_t) INT64_MAX)
+  {
+    GNUNET_break_op (0);
+    return GNUNET_SYSERR;
+  }
+  *integer_part = integer;
+  *fractional_part = frac;
+  return GNUNET_OK;
+}
+
+
+enum GNUNET_GenericReturnValue
+TMH_process_quantity_inputs (enum TMH_ValueKind kind,
+                             bool allow_fractional,
+                             bool int_missing,
+                             int64_t int_raw,
+                             bool str_missing,
+                             const char *str_raw,
+                             uint64_t *int_out,
+                             uint32_t *frac_out,
+                             const char **error_param)
+{
+  static char errbuf[128];
+  int64_t parsed_int = 0;
+  uint32_t parsed_frac = 0;
+  const char *int_field = (TMH_VK_STOCK == kind)
+                          ? "total_stock"
+                          : "quantity";
+  const char *str_field = (TMH_VK_STOCK == kind)
+                          ? "unit_total_stock"
+                          : "unit_quantity";
+
+  *error_param = NULL;
+
+  if (int_missing && str_missing)
+  {
+    GNUNET_snprintf (errbuf,
+                     sizeof (errbuf),
+                     "missing %s and %s",
+                     int_field,
+                     str_field);
+    *error_param = errbuf;
+    GNUNET_break_op (0);
+    return GNUNET_SYSERR;
+  }
+
+  if (! str_missing)
+  {
+    if ( (TMH_VK_STOCK == kind) &&
+         (0 == strcmp ("-1",
+                       str_raw)) )
+    {
+      parsed_int = -1;
+      parsed_frac = 0;
+    }
+    else
+    {
+      if (GNUNET_OK !=
+          TMH_parse_fractional_string (str_raw,
+                                       &parsed_int,
+                                       &parsed_frac))
+      {
+        GNUNET_snprintf (errbuf,
+                         sizeof (errbuf),
+                         "malformed %s",
+                         str_field);
+        *error_param = errbuf;
+        GNUNET_break_op (0);
+        return GNUNET_SYSERR;
+      }
+    }
+  }
+
+  if ( (! int_missing) && (! str_missing) )
+  {
+    if ( (parsed_int != int_raw) || (0 != parsed_frac) )
+    {
+      GNUNET_snprintf (errbuf,
+                       sizeof (errbuf),
+                       "%s/%s mismatch",
+                       int_field,
+                       str_field);
+      *error_param = errbuf;
+      GNUNET_break_op (0);
+      return GNUNET_SYSERR;
+    }
+  }
+  else if (int_missing)
+  {
+    int_raw = parsed_int;
+  }
+
+  if ( (TMH_VK_STOCK == kind) && (-1 == int_raw) )
+  {
+    if ( (! str_missing) && (0 != parsed_frac) )
+    {
+      GNUNET_snprintf (errbuf,
+                       sizeof (errbuf),
+                       "fractional part forbidden with %s='-1'",
+                       str_field);
+      *error_param = errbuf;
+      GNUNET_break_op (0);
+      return GNUNET_SYSERR;
+    }
+    *int_out = INT64_MAX;
+    *frac_out = INT32_MAX;
+    return GNUNET_OK;
+  }
+
+  if (int_raw < 0)
+  {
+    GNUNET_snprintf (errbuf,
+                     sizeof (errbuf),
+                     "%s must be non-negative",
+                     int_field);
+    *error_param = errbuf;
+    GNUNET_break_op (0);
+    return GNUNET_SYSERR;
+  }
+
+  if (! allow_fractional)
+  {
+    if ( (! str_missing) && (0 != parsed_frac) )
+    {
+      GNUNET_snprintf (errbuf,
+                       sizeof (errbuf),
+                       "fractional part not allowed for %s",
+                       str_field);
+      *error_param = errbuf;
+      GNUNET_break_op (0);
+      return GNUNET_SYSERR;
+    }
+    parsed_frac = 0;
+  }
+  else if (! str_missing)
+  {
+    if (parsed_frac >= MERCHANT_UNIT_FRAC_BASE)
+    {
+      GNUNET_snprintf (errbuf,
+                       sizeof (errbuf),
+                       "%s fractional part exceeds base %u",
+                       str_field,
+                       MERCHANT_UNIT_FRAC_BASE);
+      *error_param = errbuf;
+      GNUNET_break_op (0);
+      return GNUNET_SYSERR;
+    }
+  }
+
+  *int_out = (uint64_t) int_raw;
+  *frac_out = parsed_frac;
+  return GNUNET_OK;
+}
+
+
+void
+TMH_format_fractional_string (enum TMH_ValueKind kind,
+                              uint64_t integer,
+                              uint32_t fractional,
+                              size_t buffer_length,
+                              char buffer[static buffer_length])
+{
+  GNUNET_assert (0 < buffer_length);
+
+  if ( (TMH_VK_STOCK == kind) &&
+       (INT64_MAX == (int64_t) integer) &&
+       (INT32_MAX == (int32_t) fractional) )
+  {
+    GNUNET_snprintf (buffer,
+                     buffer_length,
+                     "-1");
+    return;
+  }
+
+  GNUNET_assert ( (TMH_VK_QUANTITY != kind) ||
+                  ((INT64_MAX != (int64_t) integer) &&
+                   (INT32_MAX != (int32_t) fractional)) );
+  GNUNET_assert (fractional < MERCHANT_UNIT_FRAC_BASE);
+
+  if (0 == fractional)
+  {
+    GNUNET_snprintf (buffer,
+                     buffer_length,
+                     "%lu",
+                     integer);
+    return;
+  }
+  {
+    char frac_buf[MERCHANT_UNIT_FRAC_MAX_DIGITS + 1];
+    size_t idx;
+
+    GNUNET_snprintf (frac_buf,
+                     sizeof (frac_buf),
+                     "%0*u",
+                     MERCHANT_UNIT_FRAC_MAX_DIGITS,
+                     (unsigned int) fractional);
+    for (idx = strlen (frac_buf); idx > 0; idx--)
+    {
+      if ('0' != frac_buf[idx - 1])
+        break;
+      frac_buf[idx - 1] = '\0';
+    }
+    GNUNET_snprintf (buffer,
+                     buffer_length,
+                     "%lu.%s",
+                     integer,
+                     frac_buf);
+  }
+}
+
+
+void
+TMH_quantity_defaults_from_unit (const struct TMH_MerchantInstance *mi,
+                                 const char *unit,
+                                 bool *allow_fractional,
+                                 uint32_t *precision_level)
+{
+  GNUNET_assert (NULL != allow_fractional);
+  GNUNET_assert (NULL != precision_level);
+  if (GNUNET_OK !=
+      TMH_unit_defaults_for_instance (mi,
+                                      unit,
+                                      allow_fractional,
+                                      precision_level))
+  {
+    *allow_fractional = false;
+    *precision_level = 0;
+  }
+}
+
+
+enum GNUNET_GenericReturnValue
+TMH_unit_defaults_for_instance (const struct TMH_MerchantInstance *mi,
+                                const char *unit,
+                                bool *allow_fractional,
+                                uint32_t *precision_level)
+{
+  struct TALER_MERCHANTDB_UnitDetails ud = { 0 };
+  enum GNUNET_DB_QueryStatus qs;
+  bool allow = false;
+  uint32_t precision = 0;
+
+  GNUNET_assert (NULL != allow_fractional);
+  GNUNET_assert (NULL != precision_level);
+
+  qs = TMH_db->select_unit (TMH_db->cls,
+                            mi->settings.id,
+                            unit,
+                            &ud);
+  switch (qs)
+  {
+  case GNUNET_DB_STATUS_HARD_ERROR:
+  case GNUNET_DB_STATUS_SOFT_ERROR:
+    TALER_MERCHANTDB_unit_details_free (&ud);
+    return GNUNET_SYSERR;
+  case GNUNET_DB_STATUS_SUCCESS_ONE_RESULT:
+    allow = ud.unit_allow_fraction;
+    precision = ud.unit_precision_level;
+    break;
+  case GNUNET_DB_STATUS_SUCCESS_NO_RESULTS:
+    break;
+  default:
+    GNUNET_break (0);
+    TALER_MERCHANTDB_unit_details_free (&ud);
+    return GNUNET_SYSERR;
+  }
+  TALER_MERCHANTDB_unit_details_free (&ud);
+
+  /* This is definitely not supposed to happen
+     combination of allow -> false, and precision > 0
+     yet let's fix it */
+  if (! allow)
+  {
+    GNUNET_break (0);
+    precision = 0;
+  }
+  *allow_fractional = allow;
+  *precision_level = precision;
+  return GNUNET_OK;
+}
+
+
 enum GNUNET_GenericReturnValue
 TMH_cmp_wire_account (
   const json_t *account,
@@ -307,6 +666,9 @@ TMH_products_array_valid (const json_t *products)
     const char *product_id = NULL;
     const char *description;
     uint64_t quantity = 0;
+    bool quantity_missing = true;
+    const char *unit_quantity = NULL;
+    bool unit_quantity_missing = true;
     const char *unit = NULL;
     struct TALER_Amount price = { .value = 0 };
     const char *image_data_url = NULL;
@@ -322,7 +684,11 @@ TMH_products_array_valid (const json_t *products)
       GNUNET_JSON_spec_mark_optional (
         GNUNET_JSON_spec_uint64 ("quantity",
                                  &quantity),
-        NULL),
+        &quantity_missing),
+      GNUNET_JSON_spec_mark_optional (
+        GNUNET_JSON_spec_string ("unit_quantity",
+                                 &unit_quantity),
+        &unit_quantity_missing),
       GNUNET_JSON_spec_mark_optional (
         GNUNET_JSON_spec_string ("unit",
                                  &unit),
@@ -360,6 +726,14 @@ TMH_products_array_valid (const json_t *products)
                   ename);
       return false;
     }
+    if (quantity_missing)
+    {
+      if (unit_quantity_missing)
+      {
+        GNUNET_break_op (0);
+        valid = false;
+      }
+    }
     if ( (NULL != image_data_url) &&
          (! TMH_image_data_url_valid (image_data_url)) )
     {
