@@ -20,8 +20,10 @@
  * @author Benedikt Mueller
  * @author Christian Grothoff
  */
-#ifndef TALER_EXCHANGE_HTTPD_HELPER_H
-#define TALER_EXCHANGE_HTTPD_HELPER_H
+#ifndef TALER_MERCHANT_HTTPD_HELPER_H
+#define TALER_MERCHANT_HTTPD_HELPER_H
+
+#define TMH_MAX_FRACTIONAL_PRECISION_LEVEL 6
 
 
 #include "taler-merchant-httpd.h"
@@ -72,6 +74,115 @@ bool
 TMH_products_array_valid (const json_t *products);
 
 
+/**
+ * Parse decimal quantity expressed as string for request handling.
+ *
+ * @param value string to parse
+ * @param[out] integer_part result integer component
+ * @param[out] fractional_part result fractional component (0..MERCHANT_UNIT_FRAC_BASE-1)
+ * @return #GNUNET_OK on success, #GNUNET_SYSERR on validation failure
+ */
+enum GNUNET_GenericReturnValue
+TMH_parse_fractional_string (const char *value,
+                             int64_t *integer_part,
+                             uint32_t *fractional_part);
+
+/**
+ * Kind of fixed-decimal value the helpers operate on.
+ * Primarily distinguishes how special sentinel values (such as "-1"
+ * meaning infinity for stock) must be encoded.
+ */
+enum TMH_ValueKind
+{
+  TMH_VK_QUANTITY,  /* -1 is illegal                    */
+  TMH_VK_STOCK      /* -1 means "infinity"              */
+};
+
+/**
+ * Extract a fixed-decimal number that may be supplied either
+ *   - as pure integer  (e.g. "total_stock"), or
+ *   - as decimal text (e.g. "unit_total_stock").
+ *
+ * Rules:
+ *  - If both forms are missing          -> error.
+ *  - If both are present                -> they must match and the decimal must have no fraction.
+ *  - For kind == TMH_VK_STOCK the integer value -1 represents infinity.
+ *
+ * @param kind             See #TMH_ValueKind
+ * @param allow_fractional False: any fractional part is rejected
+ * @param int_missing      True if client omitted the integer field
+ * @param int_raw          Raw integer (undefined if @a int_missing is true)
+ * @param str_missing      True if client omitted the string field
+ * @param str_raw          Raw UTF-8 string (undefined if @a str_missing is true)
+ * @param[out] int_out     Canonicalised integer part
+ * @param[out] frac_out    Canonicalised fractional part
+ * @param[out] error_param Set to offending field name on failure
+ * @param int_field        Integer field name (for error reporting)
+ * @param str_field        String field name (for error reporting)
+ * @return #GNUNET_OK on success, #GNUNET_SYSERR otherwise
+ */
+enum GNUNET_GenericReturnValue
+TMH_process_quantity_inputs (enum TMH_ValueKind kind,
+                             bool allow_fractional,
+                             bool int_missing,
+                             int64_t int_raw,
+                             bool str_missing,
+                             const char *str_raw,
+                             uint64_t *int_out,
+                             uint32_t *frac_out,
+                             const char **error_param);
+
+/**
+ * Lookup the defaults for @a unit within @a mi and fall back to sane
+ * values (disallow fractional quantities, zero precision) if no data
+ * is available.
+ *
+ * @param mi merchant instance whose defaults should be consulted (must not be NULL)
+ * @param unit textual unit name (must not be NULL or empty)
+ * @param allow_fractional updated with whether fractional quantities are allowed (must not be NULL)
+ * @param precision_level updated with the supported precision (must not be NULL)
+ */
+void
+TMH_quantity_defaults_from_unit (const struct TMH_MerchantInstance *mi,
+                                 const char *unit,
+                                 bool *allow_fractional,
+                                 uint32_t *precision_level);
+
+/**
+ * Query the database for precision defaults tied to @a unit within
+ * @a mi.  Returns #GNUNET_OK even if no unit information exists, in
+ * which case the out-parameters remain at their implicit defaults.
+ *
+ * @param mi merchant instance whose unit table is inspected (must not be NULL)
+ * @param unit textual unit name (must not be NULL or empty)
+ * @param allow_fractional updated with whether fractional quantities are allowed (must not be NULL)
+ * @param precision_level updated with the supported precision (must not be NULL)
+ * @return #GNUNET_OK on success, #GNUNET_SYSERR on database failure
+ */
+enum GNUNET_GenericReturnValue
+TMH_unit_defaults_for_instance (const struct TMH_MerchantInstance *mi,
+                                const char *unit,
+                                bool *allow_fractional,
+                                uint32_t *precision_level);
+
+/**
+ * Format a fixed-decimal pair into canonical string representation.
+ * Recognises INT64_MAX / INT32_MAX as the "-1" sentinel used for
+ * infinite stock if @a kind equals #TMH_VK_STOCK.
+ *
+ * @param kind specifies whether sentinel values are permitted
+ * @param integer integer portion
+ * @param fractional fractional portion (0..MERCHANT_UNIT_FRAC_BASE-1 or sentinel)
+ * @param buffer output buffer
+ * @param buffer_length length of @a buffer
+ */
+void
+TMH_format_fractional_string (enum TMH_ValueKind kind,
+                              uint64_t integer,
+                              uint32_t fractional,
+                              size_t buffer_length,
+                              char buffer[static buffer_length]);
+
 /**
  * Check if @a image_data_url is a valid image
  * data URL. Does not validate the actual payload,
