@@ -374,7 +374,7 @@ transmission_done_cb (void *cls,
 static void
 phase_send_challenge (struct MfaState *mfa)
 {
-  const char *prog;
+  const char *prog = NULL;
 
   switch (mfa->channel)
   {
