@@ -31,26 +31,6 @@
 #include "taler-merchant-httpd_post-orders-ID-paid.h"
 
 
-/**
- * Function called with information about a refund.
- * Sets the boolean in @a cls to true when called.
- *
- * @param cls closure, a `bool *`
- * @param coin_pub public coin from which the refund comes from
- * @param refund_amount refund amount which is being taken from @a coin_pub
- */
-static void
-refund_cb (
-  void *cls,
-  const struct TALER_CoinSpendPublicKeyP *coin_pub,
-  const struct TALER_Amount *refund_amount)
-{
-  bool *refunded = cls;
-
-  *refunded = true;
-}
-
-
 /**
  * Use database to notify other clients about the
  * session being captured.
@@ -91,11 +71,10 @@ TMH_post_orders_ID_paid (const struct TMH_RequestHandler *rh,
   const char *order_id = hc->infix;
   struct TALER_MerchantSignatureP merchant_sig;
   const char *session_id;
-  int16_t choice_index;
   struct TALER_PrivateContractHashP hct;
-  json_t *contract_terms;
-  const char *fulfillment_url;
+  char *fulfillment_url;
   enum GNUNET_DB_QueryStatus qs;
+  bool refunded;
 
   {
     struct GNUNET_JSON_Specification spec[] = {
@@ -105,15 +84,10 @@ TMH_post_orders_ID_paid (const struct TMH_RequestHandler *rh,
                                    &hct),
       GNUNET_JSON_spec_string ("session_id",
                                &session_id),
-      GNUNET_JSON_spec_mark_optional (
-        GNUNET_JSON_spec_int16 ("choice_index",
-                                &choice_index),
-        NULL),
       GNUNET_JSON_spec_end ()
     };
     enum GNUNET_GenericReturnValue res;
 
-    choice_index = -1;
     res = TALER_MHD_parse_json_data (connection,
                                      hc->request_body,
                                      spec);
@@ -139,87 +113,45 @@ TMH_post_orders_ID_paid (const struct TMH_RequestHandler *rh,
   }
 
   TMH_db->preflight (TMH_db->cls);
-  {
-    uint64_t order_serial;
 
-    qs = TMH_db->lookup_contract_terms (TMH_db->cls,
+  qs = TMH_db->update_contract_session (TMH_db->cls,
                                         hc->instance->settings.id,
-                                        order_id,
-                                        &contract_terms,
-                                        &order_serial,
-                                        NULL);
-  }
-  if (0 > qs)
-  {
-    /* single, read-only SQL statements should never cause
-       serialization problems */
-    GNUNET_break (GNUNET_DB_STATUS_SOFT_ERROR != qs);
-    /* Always report on hard error as well to enable diagnostics */
-    GNUNET_break (GNUNET_DB_STATUS_HARD_ERROR == qs);
-    return TALER_MHD_reply_with_error (connection,
-                                       MHD_HTTP_INTERNAL_SERVER_ERROR,
-                                       TALER_EC_GENERIC_DB_FETCH_FAILED,
-                                       "lookup_contract_terms");
-  }
-  if (GNUNET_DB_STATUS_SUCCESS_NO_RESULTS == qs)
+                                        &hct,
+                                        session_id,
+                                        &fulfillment_url,
+                                        &refunded);
+  switch (qs)
   {
+  case GNUNET_DB_STATUS_SUCCESS_NO_RESULTS:
     GNUNET_log (GNUNET_ERROR_TYPE_INFO,
                 "Unknown order id given: `%s'\n",
                 order_id);
     return TALER_MHD_reply_with_error (connection,
                                        MHD_HTTP_NOT_FOUND,
                                        TALER_EC_MERCHANT_GENERIC_ORDER_UNKNOWN,
-                                       order_id);
+                                       NULL);
+  case GNUNET_DB_STATUS_HARD_ERROR:
+    GNUNET_break (0);
+    return TALER_MHD_reply_with_error (connection,
+                                       MHD_HTTP_INTERNAL_SERVER_ERROR,
+                                       TALER_EC_GENERIC_DB_STORE_FAILED,
+                                       "update_contract_session");
+  case GNUNET_DB_STATUS_SOFT_ERROR:
+    GNUNET_break (0);
+    return TALER_MHD_reply_with_error (connection,
+                                       MHD_HTTP_INTERNAL_SERVER_ERROR,
+                                       TALER_EC_GENERIC_DB_STORE_FAILED,
+                                       "update_contract_session");
+  case GNUNET_DB_STATUS_SUCCESS_ONE_RESULT:
+    /* continued below */
+    break;
   }
 
-  {
-    struct TALER_PrivateContractHashP h_contract_terms;
-
-    if (GNUNET_OK !=
-        TALER_JSON_contract_hash (contract_terms,
-                                  &h_contract_terms))
-    {
-      GNUNET_break (0);
-      json_decref (contract_terms);
-      return TALER_MHD_reply_with_error (connection,
-                                         MHD_HTTP_INTERNAL_SERVER_ERROR,
-                                         TALER_EC_GENERIC_FAILED_COMPUTE_JSON_HASH,
-                                         NULL);
-    }
-    if (0 != GNUNET_memcmp (&hct,
-                            &h_contract_terms))
-    {
-      json_decref (contract_terms);
-      return TALER_MHD_reply_with_error (connection,
-                                         MHD_HTTP_BAD_REQUEST,
-                                         TALER_EC_MERCHANT_POST_ORDERS_ID_PAID_CONTRACT_HASH_MISMATCH,
-                                         NULL);
-    }
-  }
-
-  fulfillment_url
-    = json_string_value (json_object_get (contract_terms,
-                                          "fulfillment_url"));
   GNUNET_log (GNUNET_ERROR_TYPE_DEBUG,
               "Marking contract %s with %s/%s as paid\n",
               order_id,
               session_id,
               fulfillment_url);
-  qs = TMH_db->mark_contract_paid (TMH_db->cls,
-                                   hc->instance->settings.id,
-                                   &hct,
-                                   session_id,
-                                   choice_index);
-  /* If the order was paid already, we get qs == 0. */
-  if (0 > qs)
-  {
-    GNUNET_break (0);
-    json_decref (contract_terms);
-    return TALER_MHD_reply_with_error (connection,
-                                       MHD_HTTP_INTERNAL_SERVER_ERROR,
-                                       TALER_EC_GENERIC_DB_STORE_FAILED,
-                                       "mark_contract_paid");
-  }
 
   /* Wake everybody up who waits for this fulfillment_url and session_id */
   if ( (NULL != fulfillment_url) &&
@@ -227,7 +159,6 @@ TMH_post_orders_ID_paid (const struct TMH_RequestHandler *rh,
     trigger_session_notification (hc,
                                   session_id,
                                   fulfillment_url);
-
   /*Trigger webhook */
   /*Commented out until its purpose is defined
     {
@@ -254,23 +185,13 @@ TMH_post_orders_ID_paid (const struct TMH_RequestHandler *rh,
                   fulfillment_url);
     }
   }*/
+  GNUNET_free (fulfillment_url);
 
-  /* fulfillment_url is part of the contract_terms */
-  {
-    bool refunded = false;
-
-    qs = TMH_db->lookup_refunds (TMH_db->cls,
-                                 hc->instance->settings.id,
-                                 &hct,
-                                 &refund_cb,
-                                 &refunded);
-    json_decref (contract_terms);
-    return TALER_MHD_REPLY_JSON_PACK (
-      connection,
-      MHD_HTTP_OK,
-      GNUNET_JSON_pack_bool ("refunded",
-                             refunded));
-  }
+  return TALER_MHD_REPLY_JSON_PACK (
+    connection,
+    MHD_HTTP_OK,
+    GNUNET_JSON_pack_bool ("refunded",
+                           refunded));
 }
 
 
