@@ -400,33 +400,36 @@ refund_cb (void *cls,
               "Exchange refund status for coin %s is %u\n",
               TALER_B2S (&cr->coin_pub),
               hr->http_status);
-  if (MHD_HTTP_OK != hr->http_status)
+  switch (hr->http_status)
   {
+  case MHD_HTTP_OK:
+    {
+      enum GNUNET_DB_QueryStatus qs;
+
+      cr->exchange_pub = rr->details.ok.exchange_pub;
+      cr->exchange_sig = rr->details.ok.exchange_sig;
+      qs = TMH_db->insert_refund_proof (TMH_db->cls,
+                                        cr->refund_serial,
+                                        &rr->details.ok.exchange_sig,
+                                        &rr->details.ok.exchange_pub);
+      if (0 >= qs)
+      {
+        /* generally, this is relatively harmless for the merchant, but let's at
+           least log this. */
+        GNUNET_log (GNUNET_ERROR_TYPE_WARNING,
+                    "Failed to persist exchange response to /refund in database: %d\n",
+                    qs);
+      }
+      else
+      {
+        notify_refund_obtained (cr->prd);
+      }
+    }
+    break;
+  default:
     cr->exchange_code = hr->ec;
     cr->exchange_reply = json_incref ((json_t*) hr->reply);
-  }
-  else
-  {
-    enum GNUNET_DB_QueryStatus qs;
-
-    cr->exchange_pub = rr->details.ok.exchange_pub;
-    cr->exchange_sig = rr->details.ok.exchange_sig;
-    qs = TMH_db->insert_refund_proof (TMH_db->cls,
-                                      cr->refund_serial,
-                                      &rr->details.ok.exchange_sig,
-                                      &rr->details.ok.exchange_pub);
-    if (0 >= qs)
-    {
-      /* generally, this is relatively harmless for the merchant, but let's at
-         least log this. */
-      GNUNET_log (GNUNET_ERROR_TYPE_WARNING,
-                  "Failed to persist exchange response to /refund in database: %d\n",
-                  qs);
-    }
-    else
-    {
-      notify_refund_obtained (cr->prd);
-    }
+    break;
   }
   check_resume_prd (cr->prd);
 }
