@@ -25,7 +25,7 @@
 
 
 /**
- * Handle a GET "/instances/$ID" request.
+ * Handle a GET "/instances/$ID/private" request.
  *
  * @param rh context of the handler
  * @param connection the MHD connection to handle
@@ -37,5 +37,20 @@ TMH_private_get_instances_ID (const struct TMH_RequestHandler *rh,
                               struct MHD_Connection *connection,
                               struct TMH_HandlerContext *hc);
 
+
+/**
+ * Handle a GET "/management/instances/$ID" request.
+ *
+ * @param rh context of the handler
+ * @param connection the MHD connection to handle
+ * @param[in,out] hc context with further information about the request
+ * @return MHD result code
+ */
+MHD_RESULT
+TMH_private_get_instances_default_ID (const struct TMH_RequestHandler *rh,
+                                      struct MHD_Connection *connection,
+                                      struct TMH_HandlerContext *hc);
+
+
 /* end of taler-merchant-httpd_private-get-instances-ID.h */
 #endif
