@@ -652,10 +652,11 @@ phase_fetch_contract (struct GetOrderRequestContext *gorc)
   if (GNUNET_DB_STATUS_SUCCESS_ONE_RESULT == qs)
   {
     GNUNET_log (GNUNET_ERROR_TYPE_INFO,
-                "Order %s is %s (%s) according to database\n",
+                "Order %s is %s (%s) according to database, choice %d\n",
                 hc->infix,
                 gorc->paid ? "paid" : "unpaid",
-                gorc->wired ? "wired" : "unwired");
+                gorc->wired ? "wired" : "unwired",
+                (int) gorc->choice_index);
     gorc->phase++;
     return;
   }
@@ -746,7 +747,8 @@ phase_parse_contract (struct GetOrderRequestContext *gorc)
         GNUNET_break (0);
         phase_end (gorc,
                    TALER_MHD_reply_with_error (
-                     gorc->sc.con, MHD_HTTP_INTERNAL_SERVER_ERROR,
+                     gorc->sc.con,
+                     MHD_HTTP_INTERNAL_SERVER_ERROR,
                      TALER_EC_GENERIC_DB_INVARIANT_FAILURE,
                      NULL));
         return;
@@ -757,8 +759,9 @@ phase_parse_contract (struct GetOrderRequestContext *gorc)
     }
     else
     {
-      GNUNET_log (GNUNET_ERROR_TYPE_WARNING,
-                  "choice index %i for order %s is invalid or not yet available",
+      GNUNET_break (gorc->order_only);
+      GNUNET_log (GNUNET_ERROR_TYPE_INFO,
+                  "Choice index %i for order %s is invalid or not yet available",
                   gorc->choice_index,
                   gorc->contract_terms->order_id);
     }
@@ -776,19 +779,18 @@ phase_parse_contract (struct GetOrderRequestContext *gorc)
     }
   }
 
-  if (! gorc->order_only && GNUNET_OK !=
-      TALER_JSON_contract_hash (gorc->contract_terms_json,
-                                &gorc->h_contract_terms))
+  if ( (! gorc->order_only) &&
+       (GNUNET_OK !=
+        TALER_JSON_contract_hash (gorc->contract_terms_json,
+                                  &gorc->h_contract_terms)) )
   {
-    {
-      GNUNET_break (0);
-      phase_end (gorc,
-                 TALER_MHD_reply_with_error (gorc->sc.con,
-                                             MHD_HTTP_INTERNAL_SERVER_ERROR,
-                                             TALER_EC_GENERIC_FAILED_COMPUTE_JSON_HASH,
-                                             NULL));
-      return;
-    }
+    GNUNET_break (0);
+    phase_end (gorc,
+               TALER_MHD_reply_with_error (gorc->sc.con,
+                                           MHD_HTTP_INTERNAL_SERVER_ERROR,
+                                           TALER_EC_GENERIC_FAILED_COMPUTE_JSON_HASH,
+                                           NULL));
+    return;
   }
   GNUNET_assert (NULL != gorc->contract_terms_json);
   GNUNET_assert (NULL != gorc->contract_terms);
@@ -867,14 +869,14 @@ phase_check_repurchase (struct GetOrderRequestContext *gorc)
               "Running re-purchase detection for %s/%s\n",
               gorc->session_id,
               gorc->contract_terms->fulfillment_url);
-  qs = TMH_db->lookup_order_by_fulfillment (TMH_db->cls,
-                                            hc->instance->settings.id,
-                                            gorc->contract_terms->
-                                            fulfillment_url,
-                                            gorc->session_id,
-                                            TALER_EXCHANGE_YNA_NO !=
-                                            gorc->allow_refunded_for_repurchase,
-                                            &already_paid_order_id);
+  qs = TMH_db->lookup_order_by_fulfillment (
+    TMH_db->cls,
+    hc->instance->settings.id,
+    gorc->contract_terms->fulfillment_url,
+    gorc->session_id,
+    TALER_EXCHANGE_YNA_NO !=
+    gorc->allow_refunded_for_repurchase,
+    &already_paid_order_id);
   if (0 > qs)
   {
     /* single, read-only SQL statements should never cause
@@ -942,7 +944,9 @@ phase_check_repurchase (struct GetOrderRequestContext *gorc)
     /* undefined for unpaid v1 contracts */
     GNUNET_JSON_pack_allow_null (
       TALER_JSON_pack_amount ("total_amount",
-                              &gorc->contract_amount)),
+                              TALER_amount_is_valid (&gorc->contract_amount)
+                              ? &gorc->contract_amount
+                              : NULL)),
     GNUNET_JSON_pack_string ("summary",
                              gorc->contract_terms->summary),
     GNUNET_JSON_pack_timestamp ("pay_deadline",
