@@ -237,10 +237,15 @@ cleanup (void *ctx)
 struct ProcessRefundsClosure
 {
   /**
-   * Place where we accumulate the refunds.
+   * Place where we accumulate the granted refunds.
    */
   struct TALER_Amount total_refund_amount;
 
+  /**
+   * Place where we accumulate the pending refunds.
+   */
+  struct TALER_Amount pending_refund_amount;
+
   /**
    * Set to an error code if something goes wrong.
    */
@@ -288,6 +293,11 @@ process_refunds_cb (void *cls,
                  TALER_amount_add (&prc->total_refund_amount,
                                    &prc->total_refund_amount,
                                    refund_amount));
+  if (pending)
+    GNUNET_assert (0 <=
+                   TALER_amount_add (&prc->pending_refund_amount,
+                                     &prc->pending_refund_amount,
+                                     refund_amount));
 }
 
 
@@ -315,6 +325,9 @@ add_order (void *cls,
   bool wired;
   struct TALER_MERCHANT_Contract *contract = NULL;
   int16_t choice_index = -1;
+  struct ProcessRefundsClosure prc = {
+    .ec = TALER_EC_NONE
+  };
 
   GNUNET_log (GNUNET_ERROR_TYPE_INFO,
               "Adding order `%s' (%llu) to result set at instance `%s'\n",
@@ -402,13 +415,8 @@ add_order (void *cls,
     goto cleanup;
   }
 
-  if (GNUNET_TIME_absolute_is_future (
-        contract->refund_deadline.abs_time) &&
-      paid)
+  if (paid)
   {
-    struct ProcessRefundsClosure prc = {
-      .ec = TALER_EC_NONE
-    };
     const struct TALER_Amount *brutto;
 
     switch (contract->version)
@@ -429,10 +437,13 @@ add_order (void *cls,
       GNUNET_break (0);
       goto cleanup;
     }
-
     GNUNET_assert (GNUNET_OK ==
                    TALER_amount_set_zero (brutto->currency,
                                           &prc.total_refund_amount));
+    GNUNET_assert (GNUNET_OK ==
+                   TALER_amount_set_zero (brutto->currency,
+                                          &prc.pending_refund_amount));
+
     qs = TMH_db->lookup_refunds_detailed (TMH_db->cls,
                                           po->instance_id,
                                           &h_contract_terms,
@@ -481,6 +492,21 @@ add_order (void *cls,
           TALER_JSON_pack_amount (
             "amount",
             &contract->details.v0.brutto),
+          GNUNET_JSON_pack_allow_null (
+            TALER_JSON_pack_amount (
+              "refund_amount",
+              paid
+            ? &prc.total_refund_amount
+            : NULL)),
+          GNUNET_JSON_pack_allow_null (
+            TALER_JSON_pack_amount (
+              "pending_refund_amount",
+              paid
+            ? &prc.pending_refund_amount
+            : NULL)),
+          TALER_JSON_pack_amount (
+            "amount",
+            &contract->details.v0.brutto),
           GNUNET_JSON_pack_string ("summary",
                                    contract->summary),
           GNUNET_JSON_pack_bool ("refundable",
@@ -509,6 +535,18 @@ add_order (void *cls,
                                         creation_time),
             TALER_JSON_pack_amount ("amount",
                                     &choice->amount),
+            GNUNET_JSON_pack_allow_null (
+              TALER_JSON_pack_amount (
+                "refund_amount",
+                paid
+              ? &prc.total_refund_amount
+              : NULL)),
+            GNUNET_JSON_pack_allow_null (
+              TALER_JSON_pack_amount (
+                "pending_refund_amount",
+                paid
+              ? &prc.pending_refund_amount
+              : NULL)),
             GNUNET_JSON_pack_string ("summary",
                                      contract->summary),
             GNUNET_JSON_pack_bool ("refundable",
