@@ -21,7 +21,8 @@
 #include "platform.h"
 #include "taler-merchant-httpd_private-get-products-ID.h"
 #include <taler/taler_json_lib.h>
-
+#include "taler-merchant-httpd_helper.h"
+#include <taler/taler_util.h>
 
 /**
  * Handle a GET "/products/$ID" request.
@@ -78,6 +79,19 @@ TMH_private_get_products_ID (
 
   {
     MHD_RESULT ret;
+    int64_t total_stock_api;
+    char unit_total_stock_buf[64];
+
+    if (INT64_MAX == pd.total_stock)
+      total_stock_api = -1;
+    else
+      total_stock_api = (int64_t) pd.total_stock;
+
+    TMH_format_fractional_string (TMH_VK_STOCK,
+                                  pd.total_stock,
+                                  pd.total_stock_frac,
+                                  sizeof (unit_total_stock_buf),
+                                  unit_total_stock_buf);
 
     ret = TALER_MHD_REPLY_JSON_PACK (
       connection,
@@ -101,9 +115,16 @@ TMH_private_get_products_ID (
         GNUNET_JSON_pack_array_incref ("taxes",
                                        pd.taxes)),
       GNUNET_JSON_pack_int64 ("total_stock",
-                              (INT64_MAX == pd.total_stock)
-                              ? -1LL
-                              : pd.total_stock),
+                              total_stock_api),
+      GNUNET_JSON_pack_string ("unit_total_stock",
+                               unit_total_stock_buf),
+      GNUNET_JSON_pack_bool ("unit_allow_fraction",
+                             pd.allow_fractional_quantity),
+      GNUNET_JSON_pack_uint64 ("unit_precision_level",
+                               pd.fractional_precision_level),
+      TALER_JSON_pack_amount_array ("unit_price",
+                                    pd.price_array_length,
+                                    pd.price_array),
       GNUNET_JSON_pack_uint64 ("total_sold",
                                pd.total_sold),
       GNUNET_JSON_pack_uint64 ("total_lost",
