@@ -1,6 +1,6 @@
 /*
   This file is part of TALER
-  (C) 2014-2024 Taler Systems SA
+  (C) 2014-2025 Taler Systems SA
 
   TALER is free software; you can redistribute it and/or modify it under the
   terms of the GNU Affero General Public License as published by the Free Software
@@ -30,17 +30,14 @@
  * Generate a response (array entry) based on the given arguments.
  *
  * @param cls closure with a `json_t *` array to build up the response
- * @param credit_amount how much was wired to the merchant (minus fees)
+ * @param credit_amount amount expected to be wired to the merchant (minus fees), NULL if unknown
  * @param wtid wire transfer identifier
  * @param payto_uri target account that received the wire transfer
  * @param exchange_url base URL of the exchange that made the wire transfer
  * @param transfer_serial_id serial number identifying the transfer in the backend
  * @param execution_time when did the exchange make the transfer, #GNUNET_TIME_UNIT_FOREVER_ABS
  *           if it did not yet happen
- * @param verified YES if we checked the exchange's answer and liked it,
- *                 NO if we checked the exchange's answer and it is problematic,
- *                 ALL if we did not yet check
- * @param confirmed true if the merchant acknowledged the wire transfer reception
+ * @param expected true if the merchant acknowledged the wire transfer reception
  */
 static void
 transfer_cb (void *cls,
@@ -49,9 +46,8 @@ transfer_cb (void *cls,
              struct TALER_FullPayto payto_uri,
              const char *exchange_url,
              uint64_t transfer_serial_id,
-             struct GNUNET_TIME_Timestamp execution_time,
-             bool verified,
-             bool confirmed)
+             struct GNUNET_TIME_Absolute execution_time,
+             bool expected)
 {
   json_t *ja = cls;
   json_t *r;
@@ -67,16 +63,17 @@ transfer_cb (void *cls,
                              exchange_url),
     GNUNET_JSON_pack_uint64 ("transfer_serial_id",
                              transfer_serial_id),
+    // FIXME: protocol breaking to remove...
     GNUNET_JSON_pack_bool ("verified",
-                           verified),
+                           false),
+    // FIXME: protocol breaking to remove...
     GNUNET_JSON_pack_bool ("confirmed",
-                           confirmed),
-    GNUNET_JSON_pack_allow_null (
-      GNUNET_JSON_pack_timestamp (
-        "execution_time",
-        GNUNET_TIME_absolute_is_never (execution_time.abs_time)
-        ? GNUNET_TIME_UNIT_ZERO_TS /* => field omitted */
-         : execution_time)) );
+                           true),
+    GNUNET_JSON_pack_bool ("expected",
+                           expected),
+    GNUNET_JSON_pack_timestamp (
+      "execution_time",
+      GNUNET_TIME_absolute_to_timestamp (execution_time)));
   GNUNET_assert (0 ==
                  json_array_append_new (ja,
                                         r));
@@ -103,9 +100,29 @@ TMH_private_get_transfers (const struct TMH_RequestHandler *rh,
   struct GNUNET_TIME_Timestamp after = GNUNET_TIME_UNIT_ZERO_TS;
   int64_t limit = -20;
   uint64_t offset;
-  enum TALER_EXCHANGE_YesNoAll verified;
+  enum TALER_EXCHANGE_YesNoAll expected;
 
   (void) rh;
+  TALER_MHD_parse_request_snumber (connection,
+                                   "limit",
+                                   &limit);
+  if (limit < 0)
+    offset = INT64_MAX;
+  else
+    offset = 0;
+  TALER_MHD_parse_request_number (connection,
+                                  "offset",
+                                  &offset);
+  TALER_MHD_parse_request_yna (connection,
+                               "expected",
+                               TALER_EXCHANGE_YNA_ALL,
+                               &expected);
+  TALER_MHD_parse_request_timestamp (connection,
+                                     "before",
+                                     &before);
+  TALER_MHD_parse_request_timestamp (connection,
+                                     "after",
+                                     &after);
   {
     const char *esc_payto;
 
@@ -119,66 +136,6 @@ TMH_private_get_transfers (const struct TMH_RequestHandler *rh,
       (void) MHD_http_unescape (payto_uri.full_payto);
     }
   }
-  {
-    const char *before_s;
-
-    before_s = MHD_lookup_connection_value (connection,
-                                            MHD_GET_ARGUMENT_KIND,
-                                            "before");
-    if ( (NULL != before_s) &&
-         (GNUNET_OK !=
-          GNUNET_STRINGS_fancy_time_to_timestamp (before_s,
-                                                  &before)) )
-    {
-      GNUNET_break_op (0);
-      GNUNET_free (payto_uri.full_payto);
-      return TALER_MHD_reply_with_error (connection,
-                                         MHD_HTTP_BAD_REQUEST,
-                                         TALER_EC_GENERIC_PARAMETER_MALFORMED,
-                                         "before");
-    }
-  }
-  {
-    const char *after_s;
-
-    after_s = MHD_lookup_connection_value (connection,
-                                           MHD_GET_ARGUMENT_KIND,
-                                           "after");
-    if ( (NULL != after_s) &&
-         (GNUNET_OK !=
-          GNUNET_STRINGS_fancy_time_to_timestamp (after_s,
-                                                  &after)) )
-    {
-      GNUNET_break_op (0);
-      GNUNET_free (payto_uri.full_payto);
-      return TALER_MHD_reply_with_error (connection,
-                                         MHD_HTTP_BAD_REQUEST,
-                                         TALER_EC_GENERIC_PARAMETER_MALFORMED,
-                                         "after");
-    }
-  }
-  TALER_MHD_parse_request_snumber (connection,
-                                   "limit",
-                                   &limit);
-  if (limit < 0)
-    offset = INT64_MAX;
-  else
-    offset = 0;
-  TALER_MHD_parse_request_number (connection,
-                                  "offset",
-                                  &offset);
-  if (! (TALER_MHD_arg_to_yna (connection,
-                               "verified",
-                               TALER_EXCHANGE_YNA_ALL,
-                               &verified)) )
-  {
-    GNUNET_break_op (0);
-    GNUNET_free (payto_uri.full_payto);
-    return TALER_MHD_reply_with_error (connection,
-                                       MHD_HTTP_BAD_REQUEST,
-                                       TALER_EC_GENERIC_PARAMETER_MALFORMED,
-                                       "verified");
-  }
   TMH_db->preflight (TMH_db->cls);
   {
     json_t *ja;
@@ -193,7 +150,7 @@ TMH_private_get_transfers (const struct TMH_RequestHandler *rh,
                                    after,
                                    limit,
                                    offset,
-                                   verified,
+                                   expected,
                                    &transfer_cb,
                                    ja);
     GNUNET_free (payto_uri.full_payto);
