@@ -47,6 +47,15 @@ TMH_private_patch_products_ID (
   struct TALER_MERCHANTDB_ProductDetails pd = {0};
   const json_t *categories = NULL;
   int64_t total_stock;
+  const char *unit_total_stock = NULL;
+  bool unit_total_stock_missing;
+  bool total_stock_missing;
+  bool price_missing;
+  bool unit_price_missing;
+  bool unit_allow_fraction;
+  bool unit_allow_fraction_missing;
+  uint32_t unit_precision_level;
+  bool unit_precision_missing;
   enum GNUNET_DB_QueryStatus qs;
   struct GNUNET_JSON_Specification spec[] = {
     /* new in protocol v20, thus optional for backwards-compatibility */
@@ -62,8 +71,10 @@ TMH_private_patch_products_ID (
       NULL),
     GNUNET_JSON_spec_string ("unit",
                              (const char **) &pd.unit),
-    TALER_JSON_spec_amount_any ("price",
-                                &pd.price),
+    GNUNET_JSON_spec_mark_optional (
+      TALER_JSON_spec_amount_any ("price",
+                                  &pd.price),
+      &price_missing),
     GNUNET_JSON_spec_mark_optional (
       GNUNET_JSON_spec_string ("image",
                                (const char **) &pd.image),
@@ -76,8 +87,27 @@ TMH_private_patch_products_ID (
       GNUNET_JSON_spec_array_const ("categories",
                                     &categories),
       NULL),
-    GNUNET_JSON_spec_int64 ("total_stock",
-                            &total_stock),
+    GNUNET_JSON_spec_mark_optional (
+      GNUNET_JSON_spec_string ("unit_total_stock",
+                               &unit_total_stock),
+      &unit_total_stock_missing),
+    GNUNET_JSON_spec_mark_optional (
+      GNUNET_JSON_spec_int64 ("total_stock",
+                              &total_stock),
+      &total_stock_missing),
+    GNUNET_JSON_spec_mark_optional (
+      GNUNET_JSON_spec_bool ("unit_allow_fraction",
+                             &unit_allow_fraction),
+      &unit_allow_fraction_missing),
+    GNUNET_JSON_spec_mark_optional (
+      GNUNET_JSON_spec_uint32 ("unit_precision_level",
+                               &unit_precision_level),
+      &unit_precision_missing),
+    GNUNET_JSON_spec_mark_optional (
+      TALER_JSON_spec_amount_any_array ("unit_price",
+                                        &pd.price_array_length,
+                                        &pd.price_array),
+      &unit_price_missing),
     GNUNET_JSON_spec_mark_optional (
       GNUNET_JSON_spec_uint64 ("total_lost",
                                &pd.total_lost),
@@ -124,19 +154,100 @@ TMH_private_patch_products_ID (
     if (NULL == pd.product_name)
       pd.product_name = pd.description;
   }
-  if (total_stock < -1)
+  if (! unit_price_missing)
   {
-    GNUNET_break_op (0);
-    ret = TALER_MHD_reply_with_error (connection,
-                                      MHD_HTTP_BAD_REQUEST,
-                                      TALER_EC_GENERIC_PARAMETER_MALFORMED,
-                                      "total_stock");
-    goto cleanup;
+    if (! price_missing)
+    {
+      if (0 != TALER_amount_cmp (&pd.price,
+                                 &pd.price_array[0]))
+      {
+        ret = TALER_MHD_reply_with_error (connection,
+                                          MHD_HTTP_BAD_REQUEST,
+                                          TALER_EC_GENERIC_PARAMETER_MALFORMED,
+                                          "price,unit_price mismatch");
+        goto cleanup;
+      }
+    }
+    else
+    {
+      pd.price = pd.price_array[0];
+      price_missing = false;
+    }
   }
-  if (-1 == total_stock)
-    pd.total_stock = INT64_MAX;
   else
-    pd.total_stock = (uint64_t) total_stock;
+  {
+    if (price_missing)
+    {
+      ret = TALER_MHD_reply_with_error (connection,
+                                        MHD_HTTP_BAD_REQUEST,
+                                        TALER_EC_GENERIC_PARAMETER_MALFORMED,
+                                        "price missing");
+      goto cleanup;
+    }
+    pd.price_array = GNUNET_new_array (1,
+                                       struct TALER_Amount);
+    pd.price_array[0] = pd.price;
+    pd.price_array_length = 1;
+  }
+  if (! unit_precision_missing)
+  {
+    if (unit_precision_level > TMH_MAX_FRACTIONAL_PRECISION_LEVEL)
+    {
+      ret = TALER_MHD_reply_with_error (connection,
+                                        MHD_HTTP_BAD_REQUEST,
+                                        TALER_EC_GENERIC_PARAMETER_MALFORMED,
+                                        "unit_precision_level");
+      goto cleanup;
+    }
+  }
+  {
+    bool default_allow_fractional;
+    uint32_t default_precision_level;
+
+    if (GNUNET_OK !=
+        TMH_unit_defaults_for_instance (mi,
+                                        pd.unit,
+                                        &default_allow_fractional,
+                                        &default_precision_level))
+    {
+      GNUNET_break (0);
+      ret = TALER_MHD_reply_with_error (connection,
+                                        MHD_HTTP_INTERNAL_SERVER_ERROR,
+                                        TALER_EC_GENERIC_DB_FETCH_FAILED,
+                                        "unit defaults");
+      goto cleanup;
+    }
+    if (unit_allow_fraction_missing)
+      unit_allow_fraction = default_allow_fractional;
+    if (unit_precision_missing)
+      unit_precision_level = default_precision_level;
+
+    if (! unit_allow_fraction)
+      unit_precision_level = 0;
+    pd.fractional_precision_level = unit_precision_level;
+  }
+  {
+    const char *eparam;
+    if (GNUNET_OK !=
+        TMH_process_quantity_inputs (TMH_VK_STOCK,
+                                     unit_allow_fraction,
+                                     total_stock_missing,
+                                     total_stock,
+                                     unit_total_stock_missing,
+                                     unit_total_stock,
+                                     &pd.total_stock,
+                                     &pd.total_stock_frac,
+                                     &eparam))
+    {
+      ret = TALER_MHD_reply_with_error (
+        connection,
+        MHD_HTTP_BAD_REQUEST,
+        TALER_EC_GENERIC_PARAMETER_MALFORMED,
+        eparam);
+      goto cleanup;
+    }
+    pd.allow_fractional_quantity = unit_allow_fraction;
+  }
   if (NULL == pd.address)
     pd.address = json_object ();
 
@@ -325,6 +436,7 @@ TMH_private_patch_products_ID (
                                 0);
 cleanup:
   GNUNET_free (cats);
+  GNUNET_free (pd.price_array);
   GNUNET_JSON_parse_free (spec);
   return ret;
 }
