@@ -26,6 +26,7 @@
 #include "platform.h"
 #include "taler-merchant-httpd_private-post-instances-ID-auth.h"
 #include "taler-merchant-httpd_helper.h"
+#include "taler-merchant-httpd_mfa.h"
 #include <taler/taler_json_lib.h>
 
 
@@ -41,15 +42,21 @@
  * @param mi instance to modify settings of
  * @param connection the MHD connection to handle
  * @param[in,out] hc context with further information about the request
+ * @param auth_override The authentication settings for this instance
+ *   do not apply due to administrative action. Do not check
+ *   against the DB value when updating the auth token.
+ * @param tcs set of multi-factor authorizations required
  * @return MHD result code
  */
 static MHD_RESULT
 post_instances_ID_auth (struct TMH_MerchantInstance *mi,
                         struct MHD_Connection *connection,
-                        struct TMH_HandlerContext *hc)
+                        struct TMH_HandlerContext *hc,
+                        bool auth_override,
+                        enum TEH_TanChannelSet tcs)
 {
   struct TALER_MERCHANTDB_InstanceAuthSettings ias;
-  const char *auth_token = NULL;
+  const char *auth_pw = NULL;
   json_t *jauth = hc->request_body;
 
   {
@@ -57,12 +64,82 @@ post_instances_ID_auth (struct TMH_MerchantInstance *mi,
 
     ret = TMH_check_auth_config (connection,
                                  jauth,
-                                 &auth_token);
+                                 &auth_pw);
     if (GNUNET_OK != ret)
       return (GNUNET_NO == ret) ? MHD_YES : MHD_NO;
   }
 
-  if (NULL == auth_token)
+  if ( (0 != (tcs & TEH_TCS_SMS) &&
+        ( (NULL == mi->settings.phone) ||
+          (NULL == TMH_helper_sms) ||
+          (! mi->settings.phone_validated) ) ) )
+  {
+    GNUNET_log (GNUNET_ERROR_TYPE_WARNING,
+                "Cannot reset password: SMS factor not available\n");
+    return TALER_MHD_reply_with_error (
+      connection,
+      MHD_HTTP_FORBIDDEN,
+      TALER_EC_MERCHANT_GENERIC_MFA_MISSING,
+      "phone_number");
+  }
+  if ( (0 != (tcs & TEH_TCS_EMAIL) &&
+        ( (NULL == mi->settings.email) ||
+          (NULL == TMH_helper_email) ||
+          (! mi->settings.email_validated) ) ) )
+  {
+    GNUNET_log (GNUNET_ERROR_TYPE_WARNING,
+                "Cannot reset password: E-mail factor not available\n");
+    return TALER_MHD_reply_with_error (
+      connection,
+      MHD_HTTP_FORBIDDEN,
+      TALER_EC_MERCHANT_GENERIC_MFA_MISSING,
+      "email");
+  }
+  if (! auth_override)
+  {
+    enum GNUNET_GenericReturnValue ret = GNUNET_SYSERR; // fix -Wmaybe-uninitialized
+
+    switch (tcs)
+    {
+    case TEH_TCS_NONE:
+      ret = GNUNET_OK;
+      break;
+    case TEH_TCS_SMS:
+      ret = TMH_mfa_challenges_do (hc,
+                                   TALER_MERCHANT_MFA_CO_AUTH_CONFIGURATION,
+                                   true,
+                                   TALER_MERCHANT_MFA_CHANNEL_SMS,
+                                   mi->settings.phone,
+                                   TALER_MERCHANT_MFA_CHANNEL_NONE);
+      break;
+    case TEH_TCS_EMAIL:
+      ret = TMH_mfa_challenges_do (hc,
+                                   TALER_MERCHANT_MFA_CO_AUTH_CONFIGURATION,
+                                   true,
+                                   TALER_MERCHANT_MFA_CHANNEL_EMAIL,
+                                   mi->settings.email,
+                                   TALER_MERCHANT_MFA_CHANNEL_NONE);
+      break;
+    case TEH_TCS_EMAIL_AND_SMS:
+      ret = TMH_mfa_challenges_do (hc,
+                                   TALER_MERCHANT_MFA_CO_AUTH_CONFIGURATION,
+                                   true,
+                                   TALER_MERCHANT_MFA_CHANNEL_SMS,
+                                   mi->settings.phone,
+                                   TALER_MERCHANT_MFA_CHANNEL_EMAIL,
+                                   mi->settings.email,
+                                   TALER_MERCHANT_MFA_CHANNEL_NONE);
+      break;
+    }
+    if (GNUNET_OK != ret)
+    {
+      return (GNUNET_NO == ret)
+        ? MHD_YES
+        : MHD_NO;
+    }
+  }
+
+  if (NULL == auth_pw)
   {
     memset (&ias.auth_salt,
             0,
@@ -73,7 +150,7 @@ post_instances_ID_auth (struct TMH_MerchantInstance *mi,
   }
   else
   {
-    TMH_compute_auth (auth_token,
+    TMH_compute_auth (auth_pw,
                       &ias.auth_salt,
                       &ias.auth_hash);
   }
@@ -102,6 +179,7 @@ post_instances_ID_auth (struct TMH_MerchantInstance *mi,
          to the authentication. */
       {
         struct TALER_MERCHANTDB_InstanceAuthSettings db_ias;
+        enum TALER_ErrorCode ec;
 
         qs = TMH_db->lookup_instance_auth (TMH_db->cls,
                                            mi->settings.id,
@@ -130,20 +208,23 @@ post_instances_ID_auth (struct TMH_MerchantInstance *mi,
           break;
         }
 
-        if ( (NULL == TMH_default_auth) &&
-             (! mi->auth_override) &&
-             (GNUNET_OK !=
-              TMH_check_auth (hc->auth_token,
-                              &db_ias.auth_salt,
-                              &db_ias.auth_hash)) )
+        if (! auth_override)
         {
-          TMH_db->rollback (TMH_db->cls);
-          GNUNET_log (GNUNET_ERROR_TYPE_ERROR,
-                      "Refusing auth change: old token does not match\n");
-          return TALER_MHD_reply_with_error (connection,
-                                             MHD_HTTP_UNAUTHORIZED,
-                                             TALER_EC_MERCHANT_GENERIC_UNAUTHORIZED,
-                                             NULL);
+          // FIXME are we sure what the scope here is?
+          ec = TMH_check_token (hc->auth_token,
+                                mi->settings.id,
+                                &hc->auth_scope);
+          if (TALER_EC_NONE != ec)
+          {
+            TMH_db->rollback (TMH_db->cls);
+            GNUNET_log (GNUNET_ERROR_TYPE_ERROR,
+                        "Refusing auth change: `%s'\n",
+                        TALER_ErrorCode_get_hint (ec));
+            return TALER_MHD_reply_with_error (connection,
+                                               MHD_HTTP_UNAUTHORIZED,
+                                               TALER_EC_MERCHANT_GENERIC_UNAUTHORIZED,
+                                               NULL);
+          }
         }
       }
 
@@ -180,15 +261,6 @@ retry:
     /* Finally, also update our running process */
     mi->auth = ias;
   }
-  mi->auth_override = false;
-  if (0 == strcmp (mi->settings.id,
-                   "admin"))
-  {
-    /* The default auth string should've been
-       cleared with the first request
-       for the default instance. */
-    GNUNET_assert (NULL == TMH_default_auth);
-  }
   TMH_reload_instances (mi->settings.id);
   return TALER_MHD_reply_static (connection,
                                  MHD_HTTP_NO_CONTENT,
@@ -207,30 +279,61 @@ TMH_private_post_instances_ID_auth (const struct TMH_RequestHandler *rh,
 
   return post_instances_ID_auth (mi,
                                  connection,
-                                 hc);
+                                 hc,
+                                 false,
+                                 TEH_TCS_NONE);
 }
 
 
 MHD_RESULT
-TMH_private_post_instances_default_ID_auth (const struct TMH_RequestHandler *rh,
-                                            struct MHD_Connection *connection,
-                                            struct TMH_HandlerContext *hc)
+TMH_public_post_instances_ID_auth (const struct TMH_RequestHandler *rh,
+                                   struct MHD_Connection *connection,
+                                   struct TMH_HandlerContext *hc)
+{
+  struct TMH_MerchantInstance *mi = hc->instance;
+
+  return post_instances_ID_auth (mi,
+                                 connection,
+                                 hc,
+                                 false,
+                                 TEH_mandatory_tan_channels);
+}
+
+
+MHD_RESULT
+TMH_private_post_instances_default_ID_auth (
+  const struct TMH_RequestHandler *rh,
+  struct MHD_Connection *connection,
+  struct TMH_HandlerContext *hc)
 {
   struct TMH_MerchantInstance *mi;
   MHD_RESULT ret;
 
+  if ( (NULL == hc->infix) ||
+       (0 == strcmp ("admin",
+                     hc->infix)) )
+  {
+    GNUNET_break_op (0);
+    return TALER_MHD_reply_with_error (
+      connection,
+      MHD_HTTP_FORBIDDEN,
+      TALER_EC_MERCHANT_GENERIC_MFA_MISSING,
+      "not allowed for 'admin' account");
+  }
   mi = TMH_lookup_instance (hc->infix);
   if (NULL == mi)
   {
-    return TALER_MHD_reply_with_error (connection,
-                                       MHD_HTTP_NOT_FOUND,
-                                       TALER_EC_MERCHANT_GENERIC_INSTANCE_UNKNOWN,
-                                       hc->infix);
+    return TALER_MHD_reply_with_error (
+      connection,
+      MHD_HTTP_NOT_FOUND,
+      TALER_EC_MERCHANT_GENERIC_INSTANCE_UNKNOWN,
+      hc->infix);
   }
-  mi->auth_override = true;
   ret = post_instances_ID_auth (mi,
                                 connection,
-                                hc);
+                                hc,
+                                true,
+                                TEH_TCS_NONE);
   return ret;
 }
 
