@@ -64,9 +64,10 @@ post_instances (const struct TMH_RequestHandler *rh,
   struct TMH_WireMethod *wm_head = NULL;
   struct TMH_WireMethod *wm_tail = NULL;
   const json_t *jauth;
-  bool no_transfer_delay;
   bool no_pay_delay;
   bool no_refund_delay;
+  bool no_transfer_delay;
+  bool no_rounding_interval;
   struct GNUNET_JSON_Specification spec[] = {
     GNUNET_JSON_spec_string ("id",
                              (const char **) &is.id),
@@ -96,10 +97,6 @@ post_instances (const struct TMH_RequestHandler *rh,
                            &is.jurisdiction),
     GNUNET_JSON_spec_bool ("use_stefan",
                            &is.use_stefan),
-    GNUNET_JSON_spec_mark_optional (
-      GNUNET_JSON_spec_relative_time ("default_wire_transfer_delay",
-                                      &is.default_wire_transfer_delay),
-      &no_transfer_delay),
     GNUNET_JSON_spec_mark_optional (
       GNUNET_JSON_spec_relative_time ("default_pay_delay",
                                       &is.default_pay_delay),
@@ -108,6 +105,15 @@ post_instances (const struct TMH_RequestHandler *rh,
       GNUNET_JSON_spec_relative_time ("default_refund_delay",
                                       &is.default_refund_delay),
       &no_refund_delay),
+    GNUNET_JSON_spec_mark_optional (
+      GNUNET_JSON_spec_relative_time ("default_wire_transfer_delay",
+                                      &is.default_wire_transfer_delay),
+      &no_transfer_delay),
+    GNUNET_JSON_spec_mark_optional (
+      GNUNET_JSON_spec_time_rounder_interval (
+        "default_wire_transfer_rounding_interval",
+        &is.default_wire_transfer_rounding_interval),
+      &no_rounding_interval),
     GNUNET_JSON_spec_end ()
   };
 
@@ -122,12 +128,43 @@ post_instances (const struct TMH_RequestHandler *rh,
              ? MHD_YES
              : MHD_NO;
   }
-  if (no_transfer_delay)
-    is.default_wire_transfer_delay = TMH_default_wire_transfer_delay;
   if (no_pay_delay)
     is.default_pay_delay = TMH_default_pay_delay;
   if (no_refund_delay)
     is.default_refund_delay = TMH_default_refund_delay;
+  if (no_transfer_delay)
+    is.default_wire_transfer_delay = TMH_default_wire_transfer_delay;
+  if (no_rounding_interval)
+    is.default_wire_transfer_rounding_interval
+      = TMH_default_wire_transfer_rounding_interval;
+  if (GNUNET_TIME_relative_is_forever (is.default_pay_delay))
+  {
+    GNUNET_break_op (0);
+    GNUNET_JSON_parse_free (spec);
+    return TALER_MHD_reply_with_error (connection,
+                                       MHD_HTTP_BAD_REQUEST,
+                                       TALER_EC_GENERIC_PARAMETER_MALFORMED,
+                                       "default_pay_delay");
+  }
+  if (GNUNET_TIME_relative_is_forever (is.default_refund_delay))
+  {
+    GNUNET_break_op (0);
+    GNUNET_JSON_parse_free (spec);
+    return TALER_MHD_reply_with_error (connection,
+                                       MHD_HTTP_BAD_REQUEST,
+                                       TALER_EC_GENERIC_PARAMETER_MALFORMED,
+                                       "default_refund_delay");
+  }
+  if (GNUNET_TIME_relative_is_forever (is.default_wire_transfer_delay))
+  {
+    GNUNET_break_op (0);
+    GNUNET_JSON_parse_free (spec);
+    return TALER_MHD_reply_with_error (connection,
+                                       MHD_HTTP_BAD_REQUEST,
+                                       TALER_EC_GENERIC_PARAMETER_MALFORMED,
+                                       "default_wire_transfer_delay");
+  }
+
   {
     enum GNUNET_GenericReturnValue ret;
 
