@@ -175,6 +175,7 @@ TMH_private_post_orders_ID_refund (
     enum GNUNET_DB_QueryStatus qs;
     uint64_t order_serial;
     struct GNUNET_TIME_Timestamp refund_deadline;
+    struct GNUNET_TIME_Timestamp wire_deadline;
 
     qs = TMH_db->lookup_contract_terms (TMH_db->cls,
                                         hc->instance->settings.id,
@@ -215,6 +216,8 @@ TMH_private_post_orders_ID_refund (
       struct GNUNET_JSON_Specification cspec[] = {
         GNUNET_JSON_spec_timestamp ("refund_deadline",
                                     &refund_deadline),
+        GNUNET_JSON_spec_timestamp ("wire_transfer_deadline",
+                                    &wire_deadline),
         GNUNET_JSON_spec_timestamp ("timestamp",
                                     &timestamp),
         GNUNET_JSON_spec_end ()
@@ -251,6 +254,15 @@ TMH_private_post_orders_ID_refund (
         /* NOTE: We MAY still be lucky that the exchange did not yet
            wire the funds, so we will try to give the refund anyway */
       }
+      if (GNUNET_TIME_absolute_is_past (wire_deadline.abs_time))
+      {
+        /* it is *really* too late for refunds */
+        return TALER_MHD_reply_with_error (
+          connection,
+          MHD_HTTP_GONE,
+          TALER_EC_MERCHANT_PRIVATE_POST_REFUND_AFTER_WIRE_DEADLINE,
+          NULL);
+      }
     }
   }
 
