@@ -27,7 +27,6 @@
 #include "taler-merchant-httpd_helper.h"
 #include <taler/taler_json_lib.h>
 
-
 MHD_RESULT
 TMH_private_post_products (const struct TMH_RequestHandler *rh,
                            struct MHD_Connection *connection,
@@ -38,6 +37,15 @@ TMH_private_post_products (const struct TMH_RequestHandler *rh,
   const json_t *categories = NULL;
   const char *product_id;
   int64_t total_stock;
+  const char *unit_total_stock = NULL;
+  bool unit_total_stock_missing;
+  bool total_stock_missing;
+  bool unit_price_missing;
+  bool unit_allow_fraction;
+  bool unit_allow_fraction_missing;
+  uint32_t unit_precision_level;
+  bool unit_precision_missing;
+  bool price_missing;
   struct GNUNET_JSON_Specification spec[] = {
     GNUNET_JSON_spec_string ("product_id",
                              &product_id),
@@ -54,8 +62,10 @@ TMH_private_post_products (const struct TMH_RequestHandler *rh,
       NULL),
     GNUNET_JSON_spec_string ("unit",
                              (const char **) &pd.unit),
-    TALER_JSON_spec_amount_any ("price",
-                                &pd.price),
+    GNUNET_JSON_spec_mark_optional (
+      TALER_JSON_spec_amount_any ("price",
+                                  &pd.price),
+      &price_missing),
     GNUNET_JSON_spec_mark_optional (
       GNUNET_JSON_spec_string ("image",
                                (const char **) &pd.image),
@@ -68,8 +78,27 @@ TMH_private_post_products (const struct TMH_RequestHandler *rh,
       GNUNET_JSON_spec_array_const ("categories",
                                     &categories),
       NULL),
-    GNUNET_JSON_spec_int64 ("total_stock",
-                            &total_stock),
+    GNUNET_JSON_spec_mark_optional (
+      GNUNET_JSON_spec_string ("unit_total_stock",
+                               &unit_total_stock),
+      &unit_total_stock_missing),
+    GNUNET_JSON_spec_mark_optional (
+      GNUNET_JSON_spec_int64 ("total_stock",
+                              &total_stock),
+      &total_stock_missing),
+    GNUNET_JSON_spec_mark_optional (
+      GNUNET_JSON_spec_bool ("unit_allow_fraction",
+                             &unit_allow_fraction),
+      &unit_allow_fraction_missing),
+    GNUNET_JSON_spec_mark_optional (
+      GNUNET_JSON_spec_uint32 ("unit_precision_level",
+                               &unit_precision_level),
+      &unit_precision_missing),
+    GNUNET_JSON_spec_mark_optional (
+      TALER_JSON_spec_amount_any_array ("unit_price",
+                                        &pd.price_array_length,
+                                        &pd.price_array),
+      &unit_price_missing),
     GNUNET_JSON_spec_mark_optional (
       GNUNET_JSON_spec_json ("address",
                              &pd.address),
@@ -110,15 +139,100 @@ TMH_private_post_products (const struct TMH_RequestHandler *rh,
        product name; remove once we make product_name mandatory. */
     if (NULL == pd.product_name)
       pd.product_name = pd.description;
+
+    if (! unit_price_missing)
+    {
+      if (! price_missing)
+      {
+        if (0 != TALER_amount_cmp (&pd.price,
+                                   &pd.price_array[0]))
+        {
+          ret = TALER_MHD_reply_with_error (connection,
+                                            MHD_HTTP_BAD_REQUEST,
+                                            TALER_EC_GENERIC_PARAMETER_MALFORMED,
+                                            "price,unit_price mismatch");
+          goto cleanup;
+        }
+      }
+      else
+      {
+        pd.price = pd.price_array[0];
+        price_missing = false;
+      }
+    }
+    else
+    {
+      if (price_missing)
+      {
+        ret = TALER_MHD_reply_with_error (connection,
+                                          MHD_HTTP_BAD_REQUEST,
+                                          TALER_EC_GENERIC_PARAMETER_MALFORMED,
+                                          "price and unit_price missing");
+        goto cleanup;
+      }
+      pd.price_array = GNUNET_new_array (1,
+                                         struct TALER_Amount);
+      pd.price_array[0] = pd.price;
+      pd.price_array_length = 1;
+    }
   }
-  if (total_stock < -1)
+  if (! unit_precision_missing)
   {
-    GNUNET_break_op (0);
-    ret = TALER_MHD_reply_with_error (connection,
-                                      MHD_HTTP_BAD_REQUEST,
-                                      TALER_EC_GENERIC_PARAMETER_MALFORMED,
-                                      "total_stock");
-    goto cleanup;
+    if (unit_precision_level > TMH_MAX_FRACTIONAL_PRECISION_LEVEL)
+    {
+      ret = TALER_MHD_reply_with_error (connection,
+                                        MHD_HTTP_BAD_REQUEST,
+                                        TALER_EC_GENERIC_PARAMETER_MALFORMED,
+                                        "unit_precision_level");
+      goto cleanup;
+    }
+  }
+  {
+    bool default_allow_fractional;
+    uint32_t default_precision_level;
+
+    if (GNUNET_OK !=
+        TMH_unit_defaults_for_instance (mi,
+                                        pd.unit,
+                                        &default_allow_fractional,
+                                        &default_precision_level))
+    {
+      GNUNET_break (0);
+      ret = TALER_MHD_reply_with_error (connection,
+                                        MHD_HTTP_INTERNAL_SERVER_ERROR,
+                                        TALER_EC_GENERIC_DB_FETCH_FAILED,
+                                        "unit defaults");
+      goto cleanup;
+    }
+    if (unit_allow_fraction_missing)
+      unit_allow_fraction = default_allow_fractional;
+    if (unit_precision_missing)
+      unit_precision_level = default_precision_level;
+  }
+  if (! unit_allow_fraction)
+    unit_precision_level = 0;
+  pd.fractional_precision_level = unit_precision_level;
+  {
+    const char *eparam;
+    if (GNUNET_OK !=
+        TMH_process_quantity_inputs (TMH_VK_STOCK,
+                                     unit_allow_fraction,
+                                     total_stock_missing,
+                                     total_stock,
+                                     unit_total_stock_missing,
+                                     unit_total_stock,
+                                     &pd.total_stock,
+                                     &pd.total_stock_frac,
+                                     &eparam))
+    {
+      ret = TALER_MHD_reply_with_error (
+        connection,
+        MHD_HTTP_BAD_REQUEST,
+        TALER_EC_GENERIC_PARAMETER_MALFORMED,
+        eparam);
+      goto cleanup;
+    }
+    pd.allow_fractional_quantity = unit_allow_fraction;
   }
   num_cats = json_array_size (categories);
   cats = GNUNET_new_array (num_cats,
@@ -142,11 +256,6 @@ TMH_private_post_products (const struct TMH_RequestHandler *rh,
     }
   }
 
-  if (-1 == total_stock)
-    pd.total_stock = INT64_MAX;
-  else
-    pd.total_stock = (uint64_t) total_stock;
-
   if (NULL == pd.address)
     pd.address = json_object ();
   if (NULL == pd.description_i18n)
@@ -270,6 +379,7 @@ TMH_private_post_products (const struct TMH_RequestHandler *rh,
                                 0);
 cleanup:
   GNUNET_JSON_parse_free (spec);
+  GNUNET_free (pd.price_array);
   GNUNET_free (cats);
   return ret;
 }
