@@ -88,7 +88,7 @@ TMH_PG_get_kyc_status (
            ",mk.last_rule_gen"
            ",mk.kyc_timestamp"
            ",mk.aml_review"
-           ",mk.jaccount_limits"
+           ",mk.jaccount_limits::TEXT"
            " FROM merchant_kyc mk"
            " WHERE mk.exchange_url=$3"
            "   AND mk.account_serial="
