@@ -111,7 +111,7 @@ TMH_PG_insert_contract_terms (
              " mo.order_serial"
              ",mo.merchant_serial"
              ",mo.order_id"
-             ",$3"  /* contract_terms */
+             ",$3::TEXT::JSONB"  /* contract_terms */
              ",$4"  /* h_contract_terms */
              ",mo.creation_time"
              ",$5" /* pay_deadline */
