@@ -59,7 +59,7 @@ TMH_PG_insert_exchange_account (
            ",debit_restrictions"
            ",credit_restrictions"
            ",master_sig)"
-           " VALUES ($1, $2, $3, $4, $5, $6);");
+           " VALUES ($1, $2, $3, $4::TEXT::JSONB, $5::TEXT::JSONB, $6);");
   return GNUNET_PQ_eval_prepared_non_select (pg->conn,
                                              "insert_exchange_account",
                                              params);
