@@ -51,11 +51,11 @@ TMH_PG_insert_exchange_keys (
            ",first_retry"
            ",expiration_time"
            ",exchange_url"
-           ") VALUES ($1, $2, $3, $4);");
+           ") VALUES ($1::TEXT::JSONB, $2, $3, $4);");
   PREPARE (pg,
            "update_exchange_keys",
            "UPDATE merchant_exchange_keys SET"
-           " keys_json=$1"
+           " keys_json=$1::TEXT::JSONB"
            ",first_retry=$2"
            ",expiration_time=$3"
            " WHERE"
