@@ -49,7 +49,14 @@ TMH_PG_insert_product (void *cls,
     TALER_PQ_query_param_json (pd->taxes),
     TALER_PQ_query_param_amount_with_currency (pg->conn,
                                                &pd->price),
+    TALER_PQ_query_param_array_amount_with_currency (
+      pd->price_array_length,
+      pd->price_array,
+      pg->conn),
     GNUNET_PQ_query_param_uint64 (&pd->total_stock),
+    GNUNET_PQ_query_param_uint32 (&pd->total_stock_frac),
+    GNUNET_PQ_query_param_bool (pd->allow_fractional_quantity),
+    GNUNET_PQ_query_param_uint32 (&pd->fractional_precision_level),
     TALER_PQ_query_param_json (pd->address),
     GNUNET_PQ_query_param_timestamp (&pd->next_restock),
     GNUNET_PQ_query_param_uint32 (&pd->minimum_age),
@@ -79,11 +86,11 @@ TMH_PG_insert_product (void *cls,
            "insert_product",
            "SELECT"
            " out_conflict AS conflict"
-           ",out_no_cat AS no_cat"
            ",out_no_instance AS no_instance"
+           ",out_no_cat AS no_cat"
            " FROM merchant_do_insert_product"
            "($1, $2, $3, $4::TEXT::JSONB, $5, $6, $7::TEXT::JSONB, $8"
-           ",$9, $10::TEXT::JSONB, $11, $12, $13, $14);");
+           ",$9, $10, $11, $12, $13, $14::TEXT::JSONB, $15, $16, $17, $18);");
   qs = GNUNET_PQ_eval_prepared_singleton_select (pg->conn,
                                                  "insert_product",
                                                  params,
