@@ -20,13 +20,17 @@ CREATE FUNCTION merchant_do_insert_product (
   IN in_instance_id TEXT,
   IN in_product_id TEXT,
   IN in_description TEXT,
-  IN in_description_i18n BYTEA,
+  IN in_description_i18n JSONB,
   IN in_unit TEXT,
   IN in_image TEXT,
-  IN in_taxes BYTEA,
+  IN in_taxes JSONB,
   IN in_price taler_amount_currency,
+  IN ina_price_list taler_amount_currency[],
   IN in_total_stock INT8,
-  IN in_address BYTEA,
+  IN in_total_stock_frac INT4,
+  IN in_allow_fractional_quantity BOOL,
+  IN in_fractional_precision_level INT4,
+  IN in_address JSONB,
   IN in_next_restock INT8,
   IN in_minimum_age INT4,
   IN ina_categories INT8[],
@@ -41,6 +45,8 @@ DECLARE
   my_product_serial INT8;
   i INT8;
   ini_cat INT8;
+  my_price taler_amount_currency;
+  my_price_array taler_amount_currency[];
 BEGIN
 
 -- Which instance are we using?
@@ -58,6 +64,14 @@ THEN
 END IF;
 out_no_instance=FALSE;
 
+IF COALESCE (array_length(ina_price_list,1),0) = 0
+THEN
+  my_price_array := ARRAY[in_price]::taler_amount_currency[];
+ELSE
+  my_price_array := ina_price_list;
+END IF;
+my_price := my_price_array[1];
+
 INSERT INTO merchant_inventory
  (merchant_serial
  ,product_id
@@ -66,12 +80,17 @@ INSERT INTO merchant_inventory
  ,description_i18n
  ,unit
  ,image
+ ,image_hash
  ,taxes
  ,price
+ ,price_array
  ,total_stock
+ ,total_stock_frac
+ ,allow_fractional_quantity
+ ,fractional_precision_level
  ,address
  ,next_restock
- ,minimum_age
+  ,minimum_age
 ) VALUES (
   my_merchant_id
  ,in_product_id
@@ -80,9 +99,20 @@ INSERT INTO merchant_inventory
  ,in_description_i18n
  ,in_unit
  ,in_image
+ ,CASE
+    WHEN (in_image IS NULL) OR (in_image = '')
+    THEN NULL
+    ELSE encode(public.digest(convert_to(in_image, 'UTF8'),
+                       'sha256'),
+                'hex')
+  END
  ,in_taxes
- ,in_price
+ ,my_price
+ ,my_price_array
  ,in_total_stock
+ ,in_total_stock_frac
+ ,in_allow_fractional_quantity
+ ,in_fractional_precision_level
  ,in_address
  ,in_next_restock
  ,in_minimum_age)
@@ -105,8 +135,13 @@ THEN
     AND unit=in_unit
     AND image=in_image
     AND taxes=in_taxes
-    AND price=in_price
+    AND price=my_price
+    AND to_jsonb(COALESCE(price_array, ARRAY[]::taler_amount_currency[]))
+        = to_jsonb(COALESCE(my_price_array, ARRAY[]::taler_amount_currency[]))
     AND total_stock=in_total_stock
+    AND total_stock_frac=in_total_stock_frac
+    AND allow_fractional_quantity=in_allow_fractional_quantity
+    AND fractional_precision_level=in_fractional_precision_level
     AND address=in_address
     AND next_restock=in_next_restock
     AND minimum_age=in_minimum_age;
