@@ -64,7 +64,8 @@ TMH_PG_insert_token_family (
            ",validity_granularity"
            ",start_offset"
            ",kind)"
-           " SELECT merchant_serial, $2, $3, $4, $5, $6, $7, $8, $9, $10, $11, $12"
+           " SELECT merchant_serial, $2, $3, $4, $5::TEXT::JSONB,"
+           "   $6::TEXT::JSONB, $7, $8, $9, $10, $11, $12"
            " FROM merchant_instances"
            " WHERE merchant_id=$1"
            " ON CONFLICT DO NOTHING;");
