@@ -90,14 +90,28 @@ lookup_products_cb (void *cls,
                                     &pd.unit),
       TALER_PQ_result_spec_amount_with_currency ("price",
                                                  &pd.price),
+      TALER_PQ_result_spec_array_amount_with_currency (pg->conn,
+                                                       "price_array",
+                                                       &pd.price_array_length,
+                                                       &pd.price_array),
       TALER_PQ_result_spec_json ("taxes",
                                  &pd.taxes),
       GNUNET_PQ_result_spec_uint64 ("total_stock",
                                     &pd.total_stock),
+      GNUNET_PQ_result_spec_uint32 ("total_stock_frac",
+                                    &pd.total_stock_frac),
+      GNUNET_PQ_result_spec_bool ("allow_fractional_quantity",
+                                  &pd.allow_fractional_quantity),
+      GNUNET_PQ_result_spec_uint32 ("fractional_precision_level",
+                                    &pd.fractional_precision_level),
       GNUNET_PQ_result_spec_uint64 ("total_sold",
                                     &pd.total_sold),
+      GNUNET_PQ_result_spec_uint32 ("total_sold_frac",
+                                    &pd.total_sold_frac),
       GNUNET_PQ_result_spec_uint64 ("total_lost",
                                     &pd.total_lost),
+      GNUNET_PQ_result_spec_uint32 ("total_lost_frac",
+                                    &pd.total_lost_frac),
       GNUNET_PQ_result_spec_string ("image",
                                     &pd.image),
       TALER_PQ_result_spec_json ("address",
@@ -162,10 +176,16 @@ TMH_PG_lookup_all_products (void *cls,
            ",product_name"
            ",unit"
            ",price"
+           ",price_array"
            ",taxes::TEXT"
            ",total_stock"
+           ",total_stock_frac"
+           ",allow_fractional_quantity"
+           ",fractional_precision_level"
            ",total_sold"
+           ",total_sold_frac"
            ",total_lost"
+           ",total_lost_frac"
            ",image"
            ",minv.address::TEXT"
            ",next_restock"
