@@ -126,7 +126,7 @@ TMH_PG_lookup_categories (void *cls,
            "SELECT"
            " mc.category_serial"
            ",mc.category_name"
-           ",mc.category_name_i18n"
+           ",mc.category_name_i18n::TEXT"
            ",COALESCE(COUNT(mpc.product_serial),0)"
            "   AS product_count"
            " FROM merchant_categories mc"
