@@ -78,8 +78,14 @@ TMH_PG_mark_contract_paid (
   PREPARE (pg,
            "mark_inventory_sold",
            "UPDATE merchant_inventory SET"
-           " total_sold=total_sold + order_locks.total_locked"
-           " FROM (SELECT total_locked,product_serial"
+           " total_sold = total_sold"
+           "   + order_locks.total_locked"
+           "   + ((merchant_inventory.total_sold_frac::BIGINT"
+           "       + order_locks.total_locked_frac::BIGINT) / 1000000)"
+           " ,total_sold_frac ="
+           "    ((merchant_inventory.total_sold_frac::BIGINT"
+           "      + order_locks.total_locked_frac::BIGINT) % 1000000)::INT4"
+           " FROM (SELECT total_locked,total_locked_frac,product_serial"
            "       FROM merchant_order_locks"
            "       WHERE order_serial="
            "       (SELECT order_serial"
