@@ -68,7 +68,7 @@ TMH_PG_select_account_by_uri (void *cls,
            " salt"
            ",h_wire"
            ",credit_facade_url"
-           ",credit_facade_credentials"
+           ",credit_facade_credentials::TEXT"
            ",active"
            " FROM merchant_accounts"
            " WHERE merchant_serial="
