@@ -154,7 +154,7 @@ TMH_PG_select_accounts (void *cls,
            ",ma.salt"
            ",ma.payto_uri"
            ",ma.credit_facade_url"
-           ",ma.credit_facade_credentials"
+           ",ma.credit_facade_credentials::TEXT"
            ",ma.active"
            ",mk.merchant_priv"
            ",mi.merchant_id"
