@@ -437,7 +437,7 @@ COMMENT ON PROCEDURE merchant_do_bump_amount_stat
 
 
 DROP FUNCTION IF EXISTS merchant_statistic_interval_number_get;
-CREATE OR REPLACE FUNCTION merchant_statistic_interval_number_get (
+CREATE FUNCTION merchant_statistic_interval_number_get (
   IN in_slug TEXT,
   IN in_instance_id TEXT
 )
@@ -582,7 +582,7 @@ COMMENT ON FUNCTION merchant_statistic_interval_number_get
 
 
 DROP FUNCTION IF EXISTS merchant_statistic_interval_amount_get;
-CREATE OR REPLACE FUNCTION merchant_statistic_interval_amount_get (
+CREATE FUNCTION merchant_statistic_interval_amount_get (
   IN in_slug TEXT,
   IN in_instance_id TEXT
 )
@@ -1070,3 +1070,23 @@ COMMENT ON PROCEDURE merchant_statistic_bucket_gc
   IS 'Performs garbage collection of the merchant_statistic_bucket_counter and merchant_statistic_bucket_amount tables';
 
 
+
+-- The date_trunc may not be necessary if we assume it is already truncated
+DROP FUNCTION IF EXISTS merchant_statistics_bucket_end;
+CREATE FUNCTION merchant_statistics_bucket_end (
+  IN in_bucket_start INT8,
+  IN in_range statistic_range,
+  OUT out_bucket_end INT8
+)
+LANGUAGE plpgsql
+AS $$
+BEGIN
+    IF in_range='quarter'
+    THEN
+      out_bucket_end = EXTRACT(EPOCH FROM CAST(date_trunc('quarter', to_timestamp(in_bucket_start)::date)  + interval '3 months' AS date));
+    ELSE
+      out_bucket_end = EXTRACT(EPOCH FROM CAST(to_timestamp(in_bucket_start)::date  + ('1 ' || in_range)::interval AS date));
+    END IF;
+END $$;
+COMMENT ON FUNCTION merchant_statistics_bucket_end
+IS 'computes the end time of the bucket for an event at the current time given the desired bucket range';
