@@ -52,7 +52,14 @@ TMH_PG_update_product (void *cls,
     TALER_PQ_query_param_json (pd->taxes),
     TALER_PQ_query_param_amount_with_currency (pg->conn,
                                                &pd->price), /* $8 */
-    GNUNET_PQ_query_param_uint64 (&pd->total_stock),  /* $9 */
+    TALER_PQ_query_param_array_amount_with_currency (
+      pd->price_array_length,
+      pd->price_array,
+      pg->conn), /* $9 */
+    GNUNET_PQ_query_param_uint64 (&pd->total_stock),  /* $10 */
+    GNUNET_PQ_query_param_uint32 (&pd->total_stock_frac), /* $11 */
+    GNUNET_PQ_query_param_bool (pd->allow_fractional_quantity), /* $12 */
+    GNUNET_PQ_query_param_uint32 (&pd->fractional_precision_level),
     GNUNET_PQ_query_param_uint64 (&pd->total_lost),
     TALER_PQ_query_param_json (pd->address),
     GNUNET_PQ_query_param_timestamp (&pd->next_restock),
@@ -103,7 +110,7 @@ TMH_PG_update_product (void *cls,
            ",out_no_instance AS no_instance"
            " FROM merchant_do_update_product"
            "($1,$2,$3,$4::TEXT::JSONB,$5,$6,$7::TEXT::JSONB,$8,$9"
-           ",$10,$11::TEXT::JSONB,$12,$13,$14,$15);");
+           ",$10,$11,$12,$13,$14,$15::TEXT::JSONB,$16,$17,$18,$19);");
   qs = GNUNET_PQ_eval_prepared_singleton_select (pg->conn,
                                                  "update_product",
                                                  params,
