@@ -30,13 +30,13 @@ enum GNUNET_DB_QueryStatus
 TMH_PG_update_wirewatch_progress (
   void *cls,
   const char *instance,
-  const char *payto_uri,
+  struct TALER_FullPayto payto_uri,
   uint64_t last_serial)
 {
   struct PostgresClosure *pg = cls;
   struct GNUNET_PQ_QueryParam params[] = {
     GNUNET_PQ_query_param_string (instance),
-    GNUNET_PQ_query_param_string (payto_uri),
+    GNUNET_PQ_query_param_string (payto_uri.full_payto),
     GNUNET_PQ_query_param_uint64 (&last_serial),
     GNUNET_PQ_query_param_end
   };
