@@ -50,10 +50,15 @@
 #include "pg_lookup_transfers.h"
 #include "pg_lookup_pending_deposits.h"
 #include "pg_lookup_categories.h"
+#include "pg_lookup_units.h"
 #include "pg_select_category.h"
 #include "pg_update_category.h"
 #include "pg_insert_category.h"
 #include "pg_delete_category.h"
+#include "pg_select_unit.h"
+#include "pg_insert_unit.h"
+#include "pg_update_unit.h"
+#include "pg_delete_unit.h"
 #include "pg_update_wirewatch_progress.h"
 #include "pg_select_wirewatch_accounts.h"
 #include "pg_select_open_transfers.h"
@@ -169,6 +174,7 @@
 #include "pg_lookup_statistics_counter_by_interval.h"
 #include "pg_update_donau_instance_receipts_amount.h"
 #include "pg_delete_donau_instance.h"
+#include "pg_update_contract_session.h"
 
 #ifdef HAVE_DONAU_DONAU_SERVICE_H
 #include "donau/donau_service.h"
@@ -622,6 +628,8 @@ libtaler_plugin_merchantdb_postgres_init (void *cls)
     = &TMH_PG_update_pending_webhook;
   plugin->lookup_categories
     = &TMH_PG_lookup_categories;
+  plugin->lookup_units
+    = &TMH_PG_lookup_units;
   plugin->select_category_by_name
     = &TMH_PG_select_category_by_name;
   plugin->get_kyc_status
@@ -632,12 +640,20 @@ libtaler_plugin_merchantdb_postgres_init (void *cls)
     = &TMH_PG_get_kyc_limits;
   plugin->select_category
     = &TMH_PG_select_category;
+  plugin->select_unit
+    = &TMH_PG_select_unit;
   plugin->update_category
     = &TMH_PG_update_category;
+  plugin->update_unit
+    = &TMH_PG_update_unit;
   plugin->insert_category
     = &TMH_PG_insert_category;
+  plugin->insert_unit
+    = &TMH_PG_insert_unit;
   plugin->delete_category
     = &TMH_PG_delete_category;
+  plugin->delete_unit
+    = &TMH_PG_delete_unit;
   plugin->delete_exchange_accounts
     = &TMH_PG_delete_exchange_accounts;
   plugin->select_accounts_by_exchange
@@ -658,6 +674,8 @@ libtaler_plugin_merchantdb_postgres_init (void *cls)
     = &TMH_PG_update_token_family;
   plugin->insert_token_family_key
     = &TMH_PG_insert_token_family_key;
+  plugin->update_contract_session
+    = &TMH_PG_update_contract_session;
   plugin->lookup_token_family_key
     = &TMH_PG_lookup_token_family_key;
   plugin->update_deposit_confirmation_status
