@@ -110,7 +110,11 @@ CREATE FUNCTION merchant_do_insert_product (
   IN in_image TEXT,
   IN in_taxes JSONB,
   IN in_price taler_amount_currency,
+  IN ina_price_list taler_amount_currency[],
   IN in_total_stock INT8,
+  IN in_total_stock_frac INT4,
+  IN in_allow_fractional_quantity BOOL,
+  IN in_fractional_precision_level INT4,
   IN in_address JSONB,
   IN in_next_restock INT8,
   IN in_minimum_age INT4,
@@ -126,6 +130,8 @@ DECLARE
   my_product_serial INT8;
   i INT8;
   ini_cat INT8;
+  my_price taler_amount_currency;
+  my_price_array taler_amount_currency[];
 BEGIN
 SELECT merchant_serial
   INTO my_merchant_id
@@ -139,6 +145,13 @@ THEN
   RETURN;
 END IF;
 out_no_instance=FALSE;
+IF COALESCE (array_length(ina_price_list,1),0) = 0
+THEN
+  my_price_array := ARRAY[in_price]::taler_amount_currency[];
+ELSE
+  my_price_array := ina_price_list;
+END IF;
+my_price := my_price_array[1];
 INSERT INTO merchant_inventory
  (merchant_serial
  ,product_id
@@ -150,7 +163,11 @@ INSERT INTO merchant_inventory
  ,image_hash
  ,taxes
  ,price
+ ,price_array
  ,total_stock
+ ,total_stock_frac
+ ,allow_fractional_quantity
+ ,fractional_precision_level
  ,address
  ,next_restock
   ,minimum_age
@@ -170,8 +187,12 @@ INSERT INTO merchant_inventory
                 'hex')
   END
  ,in_taxes
- ,in_price
+ ,my_price
+ ,my_price_array
  ,in_total_stock
+ ,in_total_stock_frac
+ ,in_allow_fractional_quantity
+ ,in_fractional_precision_level
  ,in_address
  ,in_next_restock
  ,in_minimum_age)
@@ -191,8 +212,13 @@ THEN
     AND unit=in_unit
     AND image=in_image
     AND taxes=in_taxes
-    AND price=in_price
+    AND price=my_price
+    AND to_jsonb(COALESCE(price_array, ARRAY[]::taler_amount_currency[]))
+        = to_jsonb(COALESCE(my_price_array, ARRAY[]::taler_amount_currency[]))
     AND total_stock=in_total_stock
+    AND total_stock_frac=in_total_stock_frac
+    AND allow_fractional_quantity=in_allow_fractional_quantity
+    AND fractional_precision_level=in_fractional_precision_level
     AND address=in_address
     AND next_restock=in_next_restock
     AND minimum_age=in_minimum_age;
@@ -574,7 +600,11 @@ CREATE FUNCTION merchant_do_update_product (
   IN in_image TEXT,
   IN in_taxes JSONB,
   IN in_price taler_amount_currency,
+  IN ina_price_list taler_amount_currency[],
   IN in_total_stock INT8,
+  IN in_total_stock_frac INT4,
+  IN in_allow_fractional_quantity BOOL,
+  IN in_fractional_precision_level INT4,
   IN in_total_lost INT8,
   IN in_address JSONB,
   IN in_next_restock INT8,
@@ -595,6 +625,8 @@ DECLARE
   i INT8;
   ini_cat INT8;
   rec RECORD;
+  my_price taler_amount_currency;
+  my_price_array taler_amount_currency[];
 BEGIN
 out_no_instance=FALSE;
 out_no_product=FALSE;
@@ -612,7 +644,9 @@ THEN
   RETURN;
 END IF;
 SELECT total_stock
+      ,total_stock_frac
       ,total_lost
+      ,allow_fractional_quantity
       ,product_serial
   INTO rec
   FROM merchant_inventory
@@ -624,6 +658,13 @@ THEN
   RETURN;
 END IF;
 my_product_serial = rec.product_serial;
+IF COALESCE (array_length(ina_price_list,1),0) = 0
+THEN
+  my_price_array := ARRAY[in_price]::taler_amount_currency[];
+ELSE
+  my_price_array := ina_price_list;
+END IF;
+my_price := my_price_array[1];
 IF rec.total_stock > in_total_stock
 THEN
   out_stocked_reduced=TRUE;
@@ -634,6 +675,14 @@ THEN
   out_lost_reduced=TRUE;
   RETURN;
 END IF;
+IF rec.allow_fractional_quantity
+   AND (NOT in_allow_fractional_quantity)
+THEN
+  DELETE
+    FROM merchant_inventory_locks
+   WHERE product_serial = my_product_serial
+     AND total_locked_frac <> 0;
+END IF;
 DELETE FROM merchant_product_categories
   WHERE product_serial=my_product_serial;
 FOR i IN 1..COALESCE(array_length(ina_categories,1),0)
@@ -666,8 +715,12 @@ UPDATE merchant_inventory SET
                            'hex')
              END
   ,taxes=in_taxes
-  ,price=in_price
+  ,price=my_price
+  ,price_array=my_price_array
   ,total_stock=in_total_stock
+  ,total_stock_frac=in_total_stock_frac
+  ,allow_fractional_quantity=in_allow_fractional_quantity
+  ,fractional_precision_level=in_fractional_precision_level
   ,total_lost=in_total_lost
   ,address=in_address
   ,next_restock=in_next_restock
