@@ -394,6 +394,26 @@ TALER_TESTING_cmd_merchant_post_products2 (
   unsigned int http_status);
 
 
+struct TALER_TESTING_Command
+TALER_TESTING_cmd_merchant_post_products3 (
+  const char *label,
+  const char *merchant_url,
+  const char *product_id,
+  const char *description,
+  json_t *description_i18n,
+  const char *unit,
+  const char *price,
+  const char *image,
+  json_t *taxes,
+  int64_t total_stock,
+  uint32_t total_stock_frac,
+  bool unit_allow_fraction,
+  uint32_t minimum_age,
+  json_t *address,
+  struct GNUNET_TIME_Timestamp next_restock,
+  unsigned int http_status);
+
+
 /**
  * Define a "POST /products" CMD, simple version
  *
@@ -459,6 +479,26 @@ TALER_TESTING_cmd_merchant_patch_product (
   unsigned int http_status);
 
 
+struct TALER_TESTING_Command
+TALER_TESTING_cmd_merchant_patch_product2 (
+  const char *label,
+  const char *merchant_url,
+  const char *product_id,
+  const char *description,
+  json_t *description_i18n,
+  const char *unit,
+  const char *price,
+  const char *image,
+  json_t *taxes,
+  int64_t total_stock,
+  uint32_t total_stock_frac,
+  bool unit_allow_fraction,
+  uint64_t total_lost,
+  json_t *address,
+  struct GNUNET_TIME_Timestamp next_restock,
+  unsigned int http_status);
+
+
 /**
  * Define a "GET /products" CMD.
  *
@@ -477,6 +517,32 @@ TALER_TESTING_cmd_merchant_get_products (const char *label,
                                          unsigned int http_status,
                                          ...);
 
+/**
+ * Expectations for fractional unit fields when checking a product.
+ */
+struct TALER_TESTING_ProductUnitExpectations
+{
+  /**
+   * Whether @e unit_allow_fraction is provided.
+   */
+  bool have_unit_allow_fraction;
+
+  /**
+   * Expected fractional flag.
+   */
+  bool unit_allow_fraction;
+
+  /**
+   * Whether @e unit_precision_level is provided.
+   */
+  bool have_unit_precision_level;
+
+  /**
+   * Expected fractional precision.
+   */
+  uint32_t unit_precision_level;
+};
+
 
 /**
  * Define a "GET product" CMD.
@@ -488,9 +554,23 @@ TALER_TESTING_cmd_merchant_get_products (const char *label,
  * @param http_status expected HTTP response code.
  * @param product_reference reference to a "POST /products" or "PATCH /products/$ID" CMD
  *        that will provide what we expect the backend to return to us
+ * @param unit_expectations optional explicit expectations for the fractional fields
  * @return the command.
  */
 struct TALER_TESTING_Command
+TALER_TESTING_cmd_merchant_get_product2 (
+  const char *label,
+  const char *merchant_url,
+  const char *product_id,
+  unsigned int http_status,
+  const char *product_reference,
+  const struct TALER_TESTING_ProductUnitExpectations *unit_expectations);
+
+
+/**
+ * Legacy helper kept for compatibility.
+ */
+struct TALER_TESTING_Command
 TALER_TESTING_cmd_merchant_get_product (const char *label,
                                         const char *merchant_url,
                                         const char *product_id,
@@ -527,6 +607,18 @@ TALER_TESTING_cmd_merchant_lock_product (
   unsigned int http_status);
 
 
+struct TALER_TESTING_Command
+TALER_TESTING_cmd_merchant_lock_product2 (
+  const char *label,
+  const char *merchant_url,
+  const char *product_id,
+  struct GNUNET_TIME_Relative duration,
+  uint32_t quantity,
+  uint32_t quantity_frac,
+  bool use_fractional_quantity,
+  unsigned int http_status);
+
+
 /**
  * Define a "DELETE product" CMD.
  *
@@ -1521,6 +1613,118 @@ TALER_TESTING_cmd_merchant_delete_otp_device (const char *label,
 
 /* ****** Templates ******* */
 
+/**
+ * Define a "POST /units" CMD.
+ *
+ * @param label command label.
+ * @param merchant_url base URL of the merchant serving the request.
+ * @param unit_id identifier of the unit.
+ * @param unit_name_long long label to store.
+ * @param unit_name_short short label to store.
+ * @param unit_allow_fraction whether fractional quantities are allowed.
+ * @param unit_precision_level fractional precision level.
+ * @param unit_active whether the unit starts active.
+ * @param unit_name_long_i18n optional translations for the long label.
+ * @param unit_name_short_i18n optional translations for the short label.
+ * @param http_status expected HTTP response code.
+ * @return the command.
+ */
+struct TALER_TESTING_Command
+TALER_TESTING_cmd_merchant_post_units (const char *label,
+                                       const char *merchant_url,
+                                       const char *unit_id,
+                                       const char *unit_name_long,
+                                       const char *unit_name_short,
+                                       bool unit_allow_fraction,
+                                       uint32_t unit_precision_level,
+                                       bool unit_active,
+                                       json_t *unit_name_long_i18n,
+                                       json_t *unit_name_short_i18n,
+                                       unsigned int http_status);
+
+
+/**
+ * Define a "PATCH /units/$ID" CMD.
+ *
+ * @param label command label.
+ * @param merchant_url base URL of the merchant serving the request.
+ * @param unit_id identifier of the unit to patch.
+ * @param unit_name_long optional new long label (NULL to skip).
+ * @param unit_name_short optional new short label (NULL to skip).
+ * @param unit_name_long_i18n optional new long label translations (NULL to skip).
+ * @param unit_name_short_i18n optional new short label translations (NULL to skip).
+ * @param unit_allow_fraction optional pointer to new fractional flag (NULL to skip).
+ * @param unit_precision_level optional pointer to new precision level (NULL to skip).
+ * @param unit_active optional pointer to new active flag (NULL to skip).
+ * @param http_status expected HTTP response code.
+ * @return the command.
+ */
+struct TALER_TESTING_Command
+TALER_TESTING_cmd_merchant_patch_unit (
+  const char *label,
+  const char *merchant_url,
+  const char *unit_id,
+  const char *unit_name_long,
+  const char *unit_name_short,
+  json_t *unit_name_long_i18n,
+  json_t *unit_name_short_i18n,
+  const bool unit_allow_fraction,
+  const uint32_t unit_precision_level,
+  const bool unit_active,
+  unsigned int http_status);
+
+
+/**
+ * Define a "GET /units" CMD.
+ *
+ * @param label command label.
+ * @param merchant_url base URL of the merchant serving the request.
+ * @param http_status expected HTTP response code.
+ * @param ... NULL-terminated list of labels (const char *) whose units must
+ *        appear in the response (verified when @a http_status is #MHD_HTTP_OK).
+ * @return the command.
+ */
+struct TALER_TESTING_Command
+TALER_TESTING_cmd_merchant_get_units (const char *label,
+                                      const char *merchant_url,
+                                      unsigned int http_status,
+                                      ...);
+
+
+/**
+ * Define a "GET /units/$ID" CMD.
+ *
+ * @param label command label.
+ * @param merchant_url base URL of the merchant serving the request.
+ * @param unit_id identifier to fetch.
+ * @param http_status expected HTTP response code.
+ * @param reference optional label of a command providing expected unit traits.
+ * @return the command.
+ */
+struct TALER_TESTING_Command
+TALER_TESTING_cmd_merchant_get_unit (const char *label,
+                                     const char *merchant_url,
+                                     const char *unit_id,
+                                     unsigned int http_status,
+                                     const char *reference);
+
+
+/**
+ * Define a "DELETE /units/$ID" CMD.
+ *
+ * @param label command label.
+ * @param merchant_url base URL of the merchant serving the request.
+ * @param unit_id identifier to delete.
+ * @param http_status expected HTTP response code.
+ * @return the command.
+ */
+struct TALER_TESTING_Command
+TALER_TESTING_cmd_merchant_delete_unit (const char *label,
+                                        const char *merchant_url,
+                                        const char *unit_id,
+                                        unsigned int http_status);
+
+
 /**
  * Define a "POST /templates" CMD.
  *
@@ -2062,8 +2266,20 @@ TALER_TESTING_cmd_merchant_get_statisticsamount (const char *label,
   op (product_description, const char) \
   op (product_image, const char) \
   op (product_stock, const int64_t) \
+  op (product_unit_total_stock, const char) \
+  op (product_unit_precision_level, const uint32_t) \
+  op (product_unit_allow_fraction, const bool) \
   op (product_unit, const char) \
   op (product_id, const char) \
+  op (unit_id, const char) \
+  op (unit_name_long, const char) \
+  op (unit_name_short, const char) \
+  op (unit_allow_fraction, const bool) \
+  op (unit_precision_level, const uint32_t) \
+  op (unit_active, const bool) \
+  op (unit_builtin, const bool) \
+  op (unit_name_long_i18n, const json_t) \
+  op (unit_name_short_i18n, const json_t) \
   op (reason, const char) \
   op (lock_uuid, const char) \
   op (auth_token, const char) \
