@@ -28,12 +28,180 @@
 #include <taler/taler_util.h>
 #include <jansson.h>
 
+/**
+ * Fixed-point base for inventory quantities (powers of 10).
+ * Six decimal digits are supported to match the maximum unit precision.
+ */
+#define MERCHANT_UNIT_FRAC_BASE 1000000U
+#define MERCHANT_UNIT_FRAC_MAX_DIGITS 6U
+
 /**
  * Return default project data used by Taler merchant.
  */
 const struct GNUNET_OS_ProjectData *
 TALER_MERCHANT_project_data (void);
 
+
+/**
+ * Channel used to transmit MFA authorization request.
+ */
+enum TALER_MERCHANT_MFA_Channel
+{
+  /**
+   * No MFA channel.
+   */
+  TALER_MERCHANT_MFA_CHANNEL_NONE = 0,
+
+  /**
+   * SMS ("sms")
+   */
+  TALER_MERCHANT_MFA_CHANNEL_SMS = 1,
+
+  /**
+   * E-mail ("email")
+   */
+  TALER_MERCHANT_MFA_CHANNEL_EMAIL,
+
+  /**
+   * TOTP ("totp"). (Not yet implemented, #10327.)
+   */
+  TALER_MERCHANT_MFA_CHANNEL_TOTP
+};
+
+
+/**
+ * Types of critical operations of a merchant backend that may require
+ * multi-factor authorization.
+ */
+enum TALER_MERCHANT_MFA_CriticalOperation
+{
+  /**
+   * Marker used to indicate that this is NOT a critical operation.
+   */
+  TALER_MERCHANT_MFA_CO_NONE = 0,
+
+  /**
+   * Instance provisioning ("instance_provision").
+   */
+  TALER_MERCHANT_MFA_CO_INSTANCE_PROVISION,
+
+  /**
+   * Bank account configuration or reconfiguration ("account_config").
+   */
+  TALER_MERCHANT_MFA_CO_ACCOUNT_CONFIGURATION,
+
+  /**
+   * Authentication configuration change ("auth_config").
+   */
+  TALER_MERCHANT_MFA_CO_AUTH_CONFIGURATION,
+
+  /**
+   * Instance deletion ("instance_deletion").
+   */
+  TALER_MERCHANT_MFA_CO_INSTANCE_DELETION,
+
+  /**
+   * Authentication token creation ("auth_token_creation").
+   */
+  TALER_MERCHANT_MFA_CO_AUTH_TOKEN_CREATION
+
+
+};
+
+
+/**
+ * Convert critical operation enumeration value to string
+ * suitable for human readers.
+ *
+ * @param co input to convert
+ * @return operation value as string
+ */
+const char *
+TALER_MERCHANT_MFA_co2s (
+  enum TALER_MERCHANT_MFA_CriticalOperation co);
+
+
+/**
+ * Convert critical operation enumeration value to string.
+ *
+ * @param co input to convert
+ * @return operation value as string
+ */
+const char *
+TALER_MERCHANT_MFA_co_to_string (
+  enum TALER_MERCHANT_MFA_CriticalOperation co);
+
+
+/**
+ * Convert string to critical operation enumeration value.
+ *
+ * @param str input to convert
+ * @return #TALER_MERCHANT_MFA_CO_NONE on failure
+ */
+enum TALER_MERCHANT_MFA_CriticalOperation
+TALER_MERCHANT_MFA_co_from_string (const char *str);
+
+
+/**
+ * Convert MFA channel enumeration value to string.
+ *
+ * @param ch input to convert
+ * @return operation value as string
+ */
+const char *
+TALER_MERCHANT_MFA_channel_to_string (
+  enum TALER_MERCHANT_MFA_Channel ch);
+
+
+/**
+ * Convert string to MFA channel enumeration value.
+ *
+ * @param str input to convert
+ * @return #TALER_MERCHANT_MFA_CHANNEL_NONE on failure
+ */
+enum TALER_MERCHANT_MFA_Channel
+TALER_MERCHANT_MFA_channel_from_string (const char *str);
+
+
+/**
+ * @brief Salted hash of a body for a request that required MFA.
+ */
+struct TALER_MERCHANT_MFA_BodyHash
+{
+  /**
+   * Hash of the body and salt.
+   */
+  struct GNUNET_ShortHashCode hash;
+};
+
+
+/**
+ * @brief Salt used when computing a `struct TALER_MERCHANT_MFA_BodyHash`
+ */
+struct TALER_MERCHANT_MFA_BodySalt
+{
+  /**
+   * Salt.
+   */
+  uint64_t salt[128 / 64];
+};
+
+
+/**
+ * Hash the given request @a body with the given @a salt to
+ * produce @a h_body for MFA checks.
+ *
+ * @param body HTTP request body, NULL if body was empty
+ * @param salt salt to use
+ * @param h_body resulting hash
+ */
+void
+TALER_MERCHANT_mfa_body_hash (
+  const json_t *body,
+  const struct TALER_MERCHANT_MFA_BodySalt *salt,
+  struct TALER_MERCHANT_MFA_BodyHash *h_body);
+
+
 /**
  * Possible versions of the contract terms.
  */
@@ -185,6 +353,7 @@ struct TALER_MERCHANT_ContractOutput
 
   union
   {
+#if FUTURE
     /**
      * Coin-based output.
      */
@@ -197,11 +366,14 @@ struct TALER_MERCHANT_ContractOutput
 
       /**
        * Base URL of the exchange that will issue the coins.
+       *
+       * NOTE: Once implemented, check if we need to allocate this here or if
+       * we again reference the JSON as we do in other places.
        */
       char *exchange_url;
 
     } coin;
-
+#endif
     /**
      * DONAU-receipt output.
      */
@@ -212,18 +384,12 @@ struct TALER_MERCHANT_ContractOutput
        */
       struct TALER_Amount amount;
 
-      /**
-       * TODO: Check if this block is possible to operate without default placeholder...
-       * Is the donation receipt required?
-       */
-      bool receipt_required;
-
       /**
        * Base URLs of the donation authorities that will issue the tax receipt.
        */
-      const char **donau_urls;
+      char **donau_urls;
 
-      /*
+      /**
        * Length of the @e donau_urls array.
        */
       unsigned int donau_urls_len;
@@ -237,6 +403,8 @@ struct TALER_MERCHANT_ContractOutput
     {
       /**
        * Slug of the token family to be issued.
+       * Note: this is a pointer into the JSON of the
+       * respective contract/request and not owned here.
        */
       const char *token_family_slug;
 
@@ -276,6 +444,17 @@ struct TALER_MERCHANT_ContractChoice
    */
   struct TALER_Amount amount;
 
+  /**
+   * Human readable description of the semantics of the choice within the
+   * contract to be shown to the user at payment.
+   */
+  char *description;
+
+  /**
+   * Map from IETF BCP 47 language tags to localized description.
+   */
+  json_t *description_i18n;
+
   /**
    * Maximum fee the merchant is willing to pay for this choice.
    * Set to an invalid amount to use instance defaults (zero or STEFAN).
@@ -774,7 +953,7 @@ TALER_MERCHANT_json_from_token_family (
  * @param families_len length of @a families array
  * @param[out] family matching token family; NULL if no result
  * @param[out] key key of matching token family; NULL if no result
- * @return #GNUNET_ERR if no matching family found; #GNUNET_OK otherwise
+ * @return #GNUNET_SYSERR if no matching family found; #GNUNET_OK otherwise
  */
 enum GNUNET_GenericReturnValue
 TALER_MERCHANT_find_token_family_key (
@@ -786,6 +965,17 @@ TALER_MERCHANT_find_token_family_key (
   struct TALER_MERCHANT_ContractTokenFamilyKey *key);
 
 
+/**
+ * Free all the fields in the given @a choice, but not @a choice itself, since
+ * it is normally part of an array.
+ *
+ * @param[in] choice contract terms choice to free
+ */
+void
+TALER_MERCHANT_contract_choice_free (
+  struct TALER_MERCHANT_ContractChoice *choice);
+
+
 /**
  * Free the @a contract and all fields in it.
  *
