@@ -27,6 +27,87 @@
 #include "merchant_api_common.h"
 #include <gnunet/gnunet_uri_lib.h>
 #include <taler/taler_json_lib.h>
+#include "taler_merchant_util.h"
+
+
+static void
+TALER_MERCHANT_format_fractional_string (uint64_t integer,
+                                         uint32_t fractional,
+                                         char *buffer,
+                                         size_t buffer_length)
+{
+  GNUNET_assert (NULL != buffer);
+  GNUNET_assert (0 < buffer_length);
+  GNUNET_assert (fractional < MERCHANT_UNIT_FRAC_BASE);
+
+  if (0 == fractional)
+  {
+    GNUNET_snprintf (buffer,
+                     buffer_length,
+                     "%lu",
+                     integer);
+    return;
+  }
+  {
+    char frac_buf[MERCHANT_UNIT_FRAC_MAX_DIGITS + 1];
+    size_t idx;
+
+    GNUNET_snprintf (frac_buf,
+                     sizeof (frac_buf),
+                     "%0*u",
+                     MERCHANT_UNIT_FRAC_MAX_DIGITS,
+                     (unsigned int) fractional);
+    for (idx = strlen (frac_buf); idx > 0; idx--)
+    {
+      if ('0' != frac_buf[idx - 1])
+        break;
+      frac_buf[idx - 1] = '\0';
+    }
+    if ('\0' == frac_buf[0])
+      GNUNET_strlcpy (frac_buf,
+                      "0",
+                      sizeof (frac_buf));
+    GNUNET_snprintf (buffer,
+                     buffer_length,
+                     "%lu.%s",
+                     integer,
+                     frac_buf);
+  }
+}
+
+
+void
+TALER_MERCHANT_format_quantity_string (uint64_t quantity,
+                                       uint32_t quantity_frac,
+                                       char *buffer,
+                                       size_t buffer_length)
+{
+  TALER_MERCHANT_format_fractional_string (quantity,
+                                           quantity_frac,
+                                           buffer,
+                                           buffer_length);
+}
+
+
+void
+TALER_MERCHANT_format_stock_string (uint64_t total_stock,
+                                    uint32_t total_stock_frac,
+                                    char *buffer,
+                                    size_t buffer_length)
+{
+  if ( (INT64_MAX == (int64_t) total_stock) &&
+       (INT32_MAX == (int32_t) total_stock_frac) )
+  {
+    GNUNET_snprintf (buffer,
+                     buffer_length,
+                     "-1");
+    return;
+  }
+  TALER_MERCHANT_format_fractional_string (total_stock,
+                                           total_stock_frac,
+                                           buffer,
+                                           buffer_length);
+}
 
 
 void
@@ -447,6 +528,8 @@ TALER_MERCHANT_handle_order_creation_response_ (
   case MHD_HTTP_GONE:
     /* The quantity of some product requested was not available. */
     {
+      bool rq_frac_missing;
+      bool aq_frac_missing;
 
       struct GNUNET_JSON_Specification spec[] = {
         GNUNET_JSON_spec_string (
@@ -455,9 +538,19 @@ TALER_MERCHANT_handle_order_creation_response_ (
         GNUNET_JSON_spec_uint64 (
           "requested_quantity",
           &por.details.gone.requested_quantity),
+        GNUNET_JSON_spec_mark_optional (
+          GNUNET_JSON_spec_uint32 (
+            "requested_quantity_frac",
+            &por.details.gone.requested_quantity_frac),
+          &rq_frac_missing),
         GNUNET_JSON_spec_uint64 (
           "available_quantity",
           &por.details.gone.available_quantity),
+        GNUNET_JSON_spec_mark_optional (
+          GNUNET_JSON_spec_uint32 (
+            "available_quantity_frac",
+            &por.details.gone.available_quantity_frac),
+          &aq_frac_missing),
         GNUNET_JSON_spec_mark_optional (
           GNUNET_JSON_spec_timestamp (
             "restock_expected",
@@ -475,6 +568,13 @@ TALER_MERCHANT_handle_order_creation_response_ (
         por.hr.http_status = 0;
         por.hr.ec = TALER_EC_GENERIC_REPLY_MALFORMED;
       }
+      else
+      {
+        if (rq_frac_missing)
+          por.details.gone.requested_quantity_frac = 0;
+        if (aq_frac_missing)
+          por.details.gone.available_quantity_frac = 0;
+      }
       break;
     }
   case MHD_HTTP_UNAVAILABLE_FOR_LEGAL_REASONS:
