@@ -117,6 +117,16 @@ handle_get_instance_finished (void *cls,
         GNUNET_JSON_spec_relative_time (
           "default_pay_delay",
           &igr.details.ok.details.default_pay_delay),
+        GNUNET_JSON_spec_mark_optional (
+          GNUNET_JSON_spec_relative_time (
+            "default_refund_delay",
+            &igr.details.ok.details.default_refund_delay),
+          NULL),
+        GNUNET_JSON_spec_mark_optional (
+          GNUNET_JSON_spec_time_rounder_interval (
+            "default_wire_transfer_rounding_interval",
+            &igr.details.ok.details.default_wire_transfer_rounding_interval),
+          NULL),
         GNUNET_JSON_spec_end ()
       };
 
