@@ -215,6 +215,10 @@ TALER_MERCHANT_instances_post (
                                default_pay_delay),
     GNUNET_JSON_pack_time_rel ("default_refund_delay",
                                default_refund_delay),
+    /* FIXME: add eventually to arguments when we transform the API... */
+    GNUNET_JSON_pack_time_rounder_interval (
+      "default_wire_transfer_rounding_interval",
+      GNUNET_TIME_RI_NONE),
     GNUNET_JSON_pack_object_steal ("auth",
                                    auth_obj));
   iph = GNUNET_new (struct TALER_MERCHANT_InstancesPostHandle);
