@@ -425,7 +425,7 @@ TALER_MERCHANT_order_pay_frontend (
   const struct TALER_MERCHANT_PaidCoin coins[static num_coins],
   unsigned int num_tokens,
   const struct TALER_MERCHANT_UsedToken tokens[static num_tokens],
-  json_t *j_output_tokens,
+  json_t *j_output_tokens, // FIXME: not used, remove?
   TALER_MERCHANT_OrderPayCallback pay_cb,
   void *pay_cb_cls)
 {
@@ -660,7 +660,7 @@ TALER_MERCHANT_order_pay (
       const struct TALER_MERCHANT_OutputToken *ev = &output_tokens[i];
 
       j_token_ev = GNUNET_JSON_PACK (
-        TALER_JSON_pack_token_envelope ("token_ev",
+        TALER_JSON_pack_token_envelope (NULL,
                                         &ev->envelope));
 
       if (0 !=
