@@ -180,9 +180,9 @@ run (void *cls,
         "set-auth-valid",
         apikey),
       TALER_TESTING_cmd_merchant_post_instances (
-        "instance-create-default",
+        "instance-create-admin",
         merchant_url,
-        "default",
+        "admin",
         MHD_HTTP_NO_CONTENT),
       TALER_TESTING_cmd_merchant_post_account (
         "instance-create-default-account",
@@ -261,9 +261,9 @@ run (void *cls,
         "set-auth-valid",
         apikey),
       TALER_TESTING_cmd_merchant_post_instances (
-        "instance-create-default",
+        "instance-create-admin",
         merchant_url,
-        "default",
+        "admin",
         MHD_HTTP_NO_CONTENT),
       TALER_TESTING_cmd_merchant_post_account (
         "instance-create-default-account",
