@@ -300,6 +300,9 @@ run (void *cls,
       "test-oauth2",
       "pass",
       MHD_HTTP_SEE_OTHER),
+    TALER_TESTING_cmd_sleep (
+      "Wait for wire transfer deadline",
+      3),
     CMD_EXEC_AGGREGATOR ("run-aggregator"),
     TALER_TESTING_cmd_check_bank_transfer (
       "check_bank_transfer-498c",
@@ -432,6 +435,9 @@ run (void *cls,
       MHD_HTTP_OK),
     TALER_TESTING_cmd_check_bank_empty (
       "check_bank_empty-aml-1"),
+    TALER_TESTING_cmd_sleep (
+      "Wait for wire transfer deadline",
+      3),
     CMD_EXEC_AGGREGATOR ("run-aggregator-aml-frozen"),
     /* AML-frozen: hence nothing happened at the bank yet: */
     TALER_TESTING_cmd_check_bank_empty (
