@@ -86,7 +86,10 @@ echo -n "."
 # NOTE: once libeufin can do long-polling, we should
 # be able to reduce the delay here and run wirewatch
 # always in the background via setup
-taler-exchange-wirewatch -L "INFO" -c "$CONF" -t &> taler-exchange-wirewatch.out
+taler-exchange-wirewatch \
+    -L "INFO" \
+    -c "$CONF" \
+    -t &> taler-exchange-wirewatch0.out
 echo -n "."
 
 taler-wallet-cli \
@@ -176,7 +179,7 @@ fi
 # CREATE ORDER AND SELL IT
 echo -n "Creating order to be paid..."
 STATUS=$(curl 'http://localhost:9966/instances/test/private/orders' \
-    -d '{"order":{"amount":"TESTKUDOS:1","summary":"payme"}}' \
+    -d '{"refund_delay":{"d_us":0}, "order":{"amount":"TESTKUDOS:1","summary":"payme"}}' \
     -w "%{http_code}" -s -o "$LAST_RESPONSE")
 
 if [ "$STATUS" != "200" ]
@@ -236,7 +239,7 @@ WIRE_DEADLINE=$(jq -r .contract_terms.wire_transfer_deadline.t_s < "$LAST_RESPON
 
 NOW=$(date +%s)
 
-TO_SLEEP=$(( WIRE_DEADLINE - NOW ))
+TO_SLEEP=$((3600 + WIRE_DEADLINE - NOW ))
 echo "waiting $TO_SLEEP secs for wire transfer"
 
 echo -n "Perform wire transfers ..."
@@ -277,7 +280,9 @@ echo -n "Notifying merchant of correct wire transfer, but on wrong instance..."
 STATUS=$(curl 'http://localhost:9966/private/transfers' \
     -d "{\"credit_amount\":\"$CREDIT_AMOUNT\",\"wtid\":\"$WTID\",\"payto_uri\":\"$TOR_PAYTO\",\"exchange_url\":\"$WURL\"}" \
     -m 3 \
-    -w "%{http_code}" -s -o "$LAST_RESPONSE")
+    -w "%{http_code}" \
+    -s \
+    -o "$LAST_RESPONSE")
 
 if [ "$STATUS" != "204" ]
 then
@@ -290,7 +295,9 @@ echo " OK"
 echo -n "Fetching wire transfers of ADMIN instance ..."
 
 STATUS=$(curl 'http://localhost:9966/private/transfers' \
-    -w "%{http_code}" -s -o "$LAST_RESPONSE")
+              -w "%{http_code}" \
+              -s \
+              -o "$LAST_RESPONSE")
 
 if [ "$STATUS" != "200" ]
 then
@@ -309,13 +316,18 @@ fi
 echo "OK"
 
 echo -n "Fetching running taler-merchant-reconciliation on bogus transfer ..."
-taler-merchant-reconciliation -c "$CONF" -L INFO -t &> taler-merchant-reconciliation-bad.log
+taler-merchant-reconciliation \
+    -c "$CONF" \
+    -L INFO \
+    -t &> taler-merchant-reconciliation0.log
 echo "OK"
 
 echo -n "Fetching wire transfers of 'test' instance ..."
 
 STATUS=$(curl 'http://localhost:9966/instances/test/private/transfers' \
-    -w "%{http_code}" -s -o "$LAST_RESPONSE")
+              -w "%{http_code}" \
+              -s \
+              -o "$LAST_RESPONSE")
 
 if [ "$STATUS" != "200" ]
 then
@@ -356,26 +368,36 @@ echo " OK"
 echo -n "Notifying merchant of correct wire transfer in the correct instance..."
 #this time in the correct instance so the order will be marked as wired...
 
-STATUS=$(curl 'http://localhost:9966/instances/test/private/transfers' \
-    -d '{"credit_amount":"'"$CREDIT_AMOUNT"'","wtid":"'"$WTID"'","payto_uri":"'"$TARGET_PAYTO"'","exchange_url":"'"$WURL"'"}' \
-    -m 3 \
-    -w "%{http_code}" -s -o "$LAST_RESPONSE")
+echo -n "Running taler-merchant-wirewatch to check transfer ..."
+taler-merchant-wirewatch \
+    -c $CONF \
+    -t \
+    -L INFO &> taler-merchant-wirewatch1.log
+echo " DONE"
 
-if [ "$STATUS" != "204" ]
-then
-    jq . < "$LAST_RESPONSE"
-    exit_fail "Expected response 204 no content, after providing transfer data. got: $STATUS"
-fi
-echo " OK"
+echo -n "Post-check for exchange deposit ..."
+taler-merchant-depositcheck \
+    -c $CONF \
+    -t \
+    -e "http://localhost:8081/" \
+    -T ${TO_SLEEP}000000 \
+    -L INFO &> depositcheck1a.log
+echo " DONE"
 
 echo -n "Fetching running taler-merchant-reconciliation on good transfer ..."
-taler-merchant-reconciliation -c $CONF -L INFO -t &> taler-merchant-reconciliation-bad.log
+taler-merchant-reconciliation \
+    -c $CONF \
+    -L INFO \
+    -T ${TO_SLEEP}000000 \
+    -t &> taler-merchant-reconciliation1.log
 echo "OK"
 
 echo -n "Fetching wire transfers of TEST instance ..."
 
 STATUS=$(curl 'http://localhost:9966/instances/test/private/transfers' \
-    -w "%{http_code}" -s -o "$LAST_RESPONSE")
+              -w "%{http_code}" \
+              -s \
+              -o "$LAST_RESPONSE")
 
 if [ "$STATUS" != "200" ]
 then
@@ -421,7 +443,7 @@ echo "================== 2nd order ====================== "
 # CREATE ORDER AND SELL IT
 echo -n "Creating 2nd order to be paid..."
 STATUS=$(curl 'http://localhost:9966/instances/test/private/orders' \
-    -d '{"order":{"amount":"TESTKUDOS:2","summary":"payme"}}' \
+    -d '{"refund_delay":{"d_us":0}, "order":{"amount":"TESTKUDOS:2","summary":"payme"}}' \
     -w "%{http_code}" -s -o "$LAST_RESPONSE")
 
 if [ "$STATUS" != "200" ]
@@ -481,20 +503,37 @@ WIRE_DEADLINE=$(jq -r .contract_terms.wire_transfer_deadline.t_s < "$LAST_RESPON
 
 NOW=$(date +%s)
 
-TO_SLEEP=$(( WIRE_DEADLINE - NOW ))
+TO_SLEEP=$((3600 + WIRE_DEADLINE - NOW ))
 echo "waiting $TO_SLEEP secs for wire transfer"
 
 echo -n "Pre-check for exchange deposit ..."
-taler-merchant-depositcheck -c $CONF -t -L INFO &> depositcheck2a.log
+taler-merchant-depositcheck \
+    -c $CONF \
+    -t \
+    -e "http://localhost:8081/" \
+    -L INFO &> depositcheck1b.log
 echo " DONE"
 
 echo -n "Perform wire transfers ..."
-taler-exchange-aggregator -y -c $CONF -T ${TO_SLEEP}000000 -t -L INFO &> aggregator2.log
-taler-exchange-transfer -c $CONF -t -L INFO &> transfer2.log
+taler-exchange-aggregator \
+    -y \
+    -c $CONF \
+    -T ${TO_SLEEP}000000 \
+    -t \
+    -L INFO &> aggregator2.log
+taler-exchange-transfer \
+    -c $CONF \
+    -t \
+    -L INFO &> transfer2.log
 echo " DONE"
 
 echo -n "Post-check for exchange deposit ..."
-taler-merchant-depositcheck -c $CONF -t -T ${TO_SLEEP}000000 -L INFO &> depositcheck2b.log
+taler-merchant-depositcheck \
+    -c $CONF \
+    -t \
+    -e "http://localhost:8081/" \
+    -T ${TO_SLEEP}000000 \
+    -L INFO &> depositcheck1c.log
 echo " DONE"
 
 
@@ -523,29 +562,27 @@ then
 fi
 echo " OK"
 
-echo -n "Notifying merchant of correct wire transfer in the correct instance..."
-#this time in the correct instance so the order will be marked as wired...
-
-STATUS=$(curl 'http://localhost:9966/instances/test/private/transfers' \
-    -d '{"credit_amount":"'"$CREDIT_AMOUNT"'","wtid":"'"$WTID"'","payto_uri":"'"$TARGET_PAYTO"'","exchange_url":"'"$WURL"'"}' \
-    -m 3 \
-    -w "%{http_code}" -s -o "$LAST_RESPONSE")
-
-if [ "$STATUS" != "204" ]
-then
-    jq . < "$LAST_RESPONSE"
-    exit_fail "Expected response 204 no content, after providing transfer data. got: $STATUS"
-fi
-echo " OK"
+echo -n "Running taler-merchant-wirewatch to check transfer ..."
+taler-merchant-wirewatch \
+    -c $CONF \
+    -t \
+    -L INFO &> taler-merchant-wirewatch2.log
+echo " DONE"
 
 echo -n "Fetching running taler-merchant-reconciliation on good transfer ..."
-taler-merchant-reconciliation -c $CONF -L INFO -t &> taler-merchant-reconciliation2.log
+taler-merchant-reconciliation \
+    -c $CONF \
+    -L INFO \
+    -T ${TO_SLEEP}000000 \
+    -t &> taler-merchant-reconciliation2.log
 echo "OK"
 
 echo -n "Fetching wire transfers of TEST instance ..."
 
 STATUS=$(curl 'http://localhost:9966/instances/test/private/transfers' \
-    -w "%{http_code}" -s -o "$LAST_RESPONSE")
+              -w "%{http_code}" \
+              -s \
+              -o "$LAST_RESPONSE")
 
 if [ "$STATUS" != "200" ]
 then
@@ -588,7 +625,7 @@ echo "================== 3rd order ====================== "
 # CREATE ORDER AND SELL IT
 echo -n "Creating 3rd order to be paid..."
 STATUS=$(curl 'http://localhost:9966/instances/test/private/orders' \
-    -d '{"order":{"amount":"TESTKUDOS:3","summary":"payme"}}' \
+    -d '{"refund_delay":{"d_us":0}, "order":{"amount":"TESTKUDOS:3","summary":"payme"}}' \
     -w "%{http_code}" -s -o "$LAST_RESPONSE")
 
 if [ "$STATUS" != "200" ]
@@ -648,26 +685,44 @@ WIRE_DEADLINE=$(jq -r .contract_terms.wire_transfer_deadline.t_s < "$LAST_RESPON
 
 NOW=$(date +%s)
 
-TO_SLEEP=$(( WIRE_DEADLINE - NOW ))
+TO_SLEEP=$((1200 + WIRE_DEADLINE - NOW ))
 echo "waiting $TO_SLEEP secs for wire transfer"
 
-echo -n "Perform wire transfers ..."
-taler-exchange-aggregator -y -c $CONF -T ${TO_SLEEP}000000 -t -L INFO &> aggregator3.log
-taler-exchange-transfer -c $CONF -t -L INFO &> transfer3.log
+echo -n "Perform wire transfers for 3rd order..."
+taler-exchange-aggregator \
+    -y \
+    -c $CONF \
+    -T ${TO_SLEEP}000000 \
+    -t \
+    -L INFO &> aggregator3.log
+taler-exchange-transfer \
+    -c $CONF \
+    -t \
+    -L INFO &> transfer3.log
 echo " DONE"
 
 echo -n "Running taler-merchant-wirewatch to check transfer ..."
-taler-merchant-wirewatch -c $CONF -t -L INFO &> taler-merchant-wirewatch.log
+taler-merchant-wirewatch \
+    -c $CONF \
+    -t \
+    -L INFO &> taler-merchant-wirewatch3.log
 echo " DONE"
 
 echo -n "Post-wirewatch check for exchange deposit ..."
-taler-merchant-depositcheck -c $CONF -t -T ${TO_SLEEP}000000 -L INFO &> depositcheck2b.log
+taler-merchant-depositcheck \
+    -c $CONF \
+    -t \
+    -e "http://localhost:8081/" \
+    -T ${TO_SLEEP}000000 \
+    -L INFO &> depositcheck1d.log
 echo " DONE"
 
 echo -n "Fetching wire transfers of TEST instance ..."
 
 STATUS=$(curl 'http://localhost:9966/instances/test/private/transfers' \
-    -w "%{http_code}" -s -o "$LAST_RESPONSE")
+              -w "%{http_code}" \
+              -s \
+              -o "$LAST_RESPONSE")
 
 if [ "$STATUS" != "200" ]
 then
@@ -686,7 +741,11 @@ fi
 echo "OK"
 
 echo -n "Fetching running taler-merchant-reconciliation on good transfer ..."
-taler-merchant-reconciliation -c $CONF -L INFO -t &> taler-merchant-reconciliation2.log
+taler-merchant-reconciliation \
+    -c $CONF \
+    -L INFO \
+    -T ${TO_SLEEP}000000 \
+    -t &> taler-merchant-reconciliation3.log
 echo "OK"
 
 echo -n "Checking order status ..."
