@@ -120,7 +120,13 @@ order_forget_cb (void *cls,
 
   ofs->ofh = NULL;
   if (ofs->http_status != hr->http_status)
-    TALER_TESTING_FAIL (ofs->is);
+  {
+    TALER_TESTING_unexpected_status_with_body (ofs->is,
+                                               hr->http_status,
+                                               ofs->http_status,
+                                               hr->reply);
+    return;
+  }
   TALER_TESTING_interpreter_next (ofs->is);
 }
 
