@@ -64,6 +64,11 @@ struct GetProductState
    */
   unsigned int http_status;
 
+  /**
+   * Optional overrides for fractional fields.
+   */
+  const struct TALER_TESTING_ProductUnitExpectations *unit_expectations;
+
 };
 
 
@@ -79,6 +84,8 @@ get_product_cb (void *cls,
 {
   struct GetProductState *gis = cls;
   const struct TALER_TESTING_Command *product_cmd;
+  const struct TALER_TESTING_ProductUnitExpectations *ue =
+    gis->unit_expectations;
 
   gis->igh = NULL;
   if (gis->http_status != pgr->hr.http_status)
@@ -148,6 +155,112 @@ get_product_cb (void *cls,
         return;
       }
     }
+    {
+      const bool *expected_allow;
+      bool have_allow = GNUNET_OK ==
+                        TALER_TESTING_get_trait_product_unit_allow_fraction (
+        product_cmd,
+        &expected_allow);
+      bool override_allow = (NULL != ue) &&
+                            ue->have_unit_allow_fraction;
+
+      if (override_allow)
+      {
+        if (pgr->details.ok.unit_allow_fraction !=
+            ue->unit_allow_fraction)
+        {
+          GNUNET_log (GNUNET_ERROR_TYPE_ERROR,
+                      "Product fractional flag does not match expectation\n");
+          TALER_TESTING_interpreter_fail (gis->is);
+          return;
+        }
+      }
+      else if (! have_allow)
+      {
+        if (pgr->details.ok.unit_allow_fraction)
+        {
+          GNUNET_log (GNUNET_ERROR_TYPE_ERROR,
+                      "Product fractional flag unexpected\n");
+          TALER_TESTING_interpreter_fail (gis->is);
+          return;
+        }
+      }
+      else
+      {
+        if (pgr->details.ok.unit_allow_fraction != *expected_allow)
+        {
+          GNUNET_log (GNUNET_ERROR_TYPE_ERROR,
+                      "Product fractional flag does not match\n");
+          TALER_TESTING_interpreter_fail (gis->is);
+          return;
+        }
+        {
+          const char *expected_unit_total_stock;
+
+          if (GNUNET_OK !=
+              TALER_TESTING_get_trait_product_unit_total_stock (
+                product_cmd,
+                &expected_unit_total_stock))
+            TALER_TESTING_interpreter_fail (gis->is);
+          else if (0 != strcmp (pgr->details.ok.unit_total_stock,
+                                expected_unit_total_stock))
+          {
+            GNUNET_log (GNUNET_ERROR_TYPE_ERROR,
+                        "Product stock string does not match\n");
+            TALER_TESTING_interpreter_fail (gis->is);
+            return;
+          }
+        }
+      }
+    }
+    {
+      const uint32_t *expected_precision;
+      bool have_precision = GNUNET_OK ==
+                            TALER_TESTING_get_trait_product_unit_precision_level (
+        product_cmd,
+        &expected_precision);
+      bool override_precision = (NULL != ue) &&
+                                ue->have_unit_precision_level;
+
+      if (override_precision)
+      {
+        if (pgr->details.ok.unit_precision_level !=
+            ue->unit_precision_level)
+        {
+          GNUNET_log (GNUNET_ERROR_TYPE_ERROR,
+                      "Product fractional precision does not match expectation\n");
+          TALER_TESTING_interpreter_fail (gis->is);
+          return;
+        }
+      }
+      else if (have_precision)
+      {
+        if (pgr->details.ok.unit_precision_level != *expected_precision)
+        {
+          GNUNET_log (GNUNET_ERROR_TYPE_ERROR,
+                      "Product fractional precision does not match\n");
+          TALER_TESTING_interpreter_fail (gis->is);
+          return;
+        }
+      }
+      else if (! pgr->details.ok.unit_allow_fraction)
+      {
+        if (0 != pgr->details.ok.unit_precision_level)
+        {
+          GNUNET_log (GNUNET_ERROR_TYPE_ERROR,
+                      "Product fractional precision should be zero when disallowed\n");
+          TALER_TESTING_interpreter_fail (gis->is);
+          return;
+        }
+      }
+      else if (pgr->details.ok.unit_precision_level > 8)
+      {
+        GNUNET_log (GNUNET_ERROR_TYPE_ERROR,
+                    "Product fractional precision exceeds supported range\n");
+        TALER_TESTING_interpreter_fail (gis->is);
+        return;
+      }
+    }
     {
       const char *expected_image;
 
@@ -203,13 +316,16 @@ get_product_cb (void *cls,
           TALER_TESTING_get_trait_address (product_cmd,
                                            &expected_location))
         TALER_TESTING_interpreter_fail (gis->is);
-      if (1 != json_equal (pgr->details.ok.location,
-                           expected_location))
+      if (NULL != expected_location)
       {
-        GNUNET_log (GNUNET_ERROR_TYPE_ERROR,
-                    "Product location does not match\n");
-        TALER_TESTING_interpreter_fail (gis->is);
-        return;
+        if (1 != json_equal (pgr->details.ok.location,
+                             expected_location))
+        {
+          GNUNET_log (GNUNET_ERROR_TYPE_ERROR,
+                      "Product location does not match\n");
+          TALER_TESTING_interpreter_fail (gis->is);
+          return;
+        }
       }
     }
     {
@@ -313,6 +429,24 @@ TALER_TESTING_cmd_merchant_get_product (const char *label,
                                         const char *product_id,
                                         unsigned int http_status,
                                         const char *product_reference)
+{
+  return TALER_TESTING_cmd_merchant_get_product2 (label,
+                                                  merchant_url,
+                                                  product_id,
+                                                  http_status,
+                                                  product_reference,
+                                                  NULL);
+}
+
+
+struct TALER_TESTING_Command
+TALER_TESTING_cmd_merchant_get_product2 (
+  const char *label,
+  const char *merchant_url,
+  const char *product_id,
+  unsigned int http_status,
+  const char *product_reference,
+  const struct TALER_TESTING_ProductUnitExpectations *unit_expectations)
 {
   struct GetProductState *gis;
 
@@ -321,6 +455,7 @@ TALER_TESTING_cmd_merchant_get_product (const char *label,
   gis->product_id = product_id;
   gis->http_status = http_status;
   gis->product_reference = product_reference;
+  gis->unit_expectations = unit_expectations;
   {
     struct TALER_TESTING_Command cmd = {
       .cls = gis,
