@@ -95,7 +95,7 @@ get_template_cb (void *cls,
   {
   case MHD_HTTP_OK:
     {
-      const char **expected_description;
+      const char *expected_description;
 
       template_cmd = TALER_TESTING_interpreter_lookup_command (
         gis->is,
@@ -104,8 +104,8 @@ get_template_cb (void *cls,
           TALER_TESTING_get_trait_template_description (template_cmd,
                                                         &expected_description))
         TALER_TESTING_interpreter_fail (gis->is);
-      if (0 != strcmp (tgr->details.success.template_description,
-                       *expected_description))
+      if (0 != strcmp (tgr->details.ok.template_description,
+                       expected_description))
       {
         GNUNET_log (GNUNET_ERROR_TYPE_ERROR,
                     "Template description does not match\n");
@@ -114,23 +114,22 @@ get_template_cb (void *cls,
       }
     }
     {
-      const char **expected_pos_key;
-      const char *pos_key = tgr->details.success.pos_key;
+      const char *expected_otp_id;
 
       if (GNUNET_OK !=
-          TALER_TESTING_get_trait_template_pos_key (template_cmd,
-                                                    &expected_pos_key))
+          TALER_TESTING_get_trait_otp_id (template_cmd,
+                                          &expected_otp_id))
         TALER_TESTING_interpreter_fail (gis->is);
-      if ( ( (NULL == pos_key) && (NULL != *expected_pos_key)) ||
-           ( (NULL != pos_key) && (NULL == *expected_pos_key)) ||
-           ( (NULL != pos_key) &&
-             (0 != strcmp (pos_key,
-                           *expected_pos_key)) ) )
+      if ( ( (NULL == tgr->details.ok.otp_id) && (NULL != expected_otp_id)) ||
+           ( (NULL != tgr->details.ok.otp_id) && (NULL == expected_otp_id)) ||
+           ( (NULL != tgr->details.ok.otp_id) &&
+             (0 != strcmp (tgr->details.ok.otp_id,
+                           expected_otp_id)) ) )
       {
         GNUNET_log (GNUNET_ERROR_TYPE_ERROR,
                     "Template pos_key `%s' does not match `%s'\n",
-                    pos_key,
-                    *expected_pos_key);
+                    tgr->details.ok.otp_id,
+                    expected_otp_id);
         TALER_TESTING_interpreter_fail (gis->is);
         return;
       }
@@ -142,7 +141,7 @@ get_template_cb (void *cls,
           TALER_TESTING_get_trait_template_contract (template_cmd,
                                                      &expected_template_contract))
         TALER_TESTING_interpreter_fail (gis->is);
-      if (1 != json_equal (tgr->details.success.template_contract,
+      if (1 != json_equal (tgr->details.ok.template_contract,
                            expected_template_contract))
       {
         GNUNET_log (GNUNET_ERROR_TYPE_ERROR,
@@ -180,11 +179,12 @@ get_template_run (void *cls,
   struct GetTemplateState *gis = cls;
 
   gis->is = is;
-  gis->igh = TALER_MERCHANT_template_get (is->ctx,
-                                          gis->merchant_url,
-                                          gis->template_id,
-                                          &get_template_cb,
-                                          gis);
+  gis->igh = TALER_MERCHANT_template_get (
+    TALER_TESTING_interpreter_get_context (is),
+    gis->merchant_url,
+    gis->template_id,
+    &get_template_cb,
+    gis);
   GNUNET_assert (NULL != gis->igh);
 }
 
