@@ -125,12 +125,13 @@ auth_instance_run (void *cls,
   struct AuthInstanceState *ais = cls;
 
   ais->is = is;
-  ais->iaph = TALER_MERCHANT_instance_auth_post (is->ctx,
-                                                 ais->merchant_url,
-                                                 ais->instance_id,
-                                                 ais->auth_token,
-                                                 &auth_instance_cb,
-                                                 ais);
+  ais->iaph = TALER_MERCHANT_instance_auth_post (
+    TALER_TESTING_interpreter_get_context (is),
+    ais->merchant_url,
+    ais->instance_id,
+    ais->auth_token,
+    &auth_instance_cb,
+    ais);
   GNUNET_assert (NULL != ais->iaph);
 }
 
@@ -175,7 +176,7 @@ auth_instance_traits (void *cls,
 {
   struct AuthInstanceState *ais = cls;
   struct TALER_TESTING_Trait traits[] = {
-    TALER_TESTING_make_trait_auth_token (&ais->auth_token),
+    TALER_TESTING_make_trait_auth_token (ais->auth_token),
     TALER_TESTING_trait_end ()
   };
 
