@@ -69,6 +69,17 @@ struct LockProductState
    */
   uint32_t quantity;
 
+  /**
+   * Fractional component of the quantity (units of 1/MERCHANT_UNIT_FRAC_BASE) when
+   * @e use_fractional_quantity is true.
+   */
+  uint32_t quantity_frac;
+
+  /**
+   * Set to true if @e quantity_frac should be sent along with @e quantity.
+   */
+  bool use_fractional_quantity;
+
   /**
    * Expected HTTP response code.
    */
@@ -135,15 +146,32 @@ lock_product_run (void *cls,
   struct LockProductState *pis = cls;
 
   pis->is = is;
-  pis->iph = TALER_MERCHANT_product_lock (
-    TALER_TESTING_interpreter_get_context (is),
-    pis->merchant_url,
-    pis->product_id,
-    pis->uuid,
-    pis->duration,
-    pis->quantity,
-    &lock_product_cb,
-    pis);
+  if (pis->use_fractional_quantity)
+  {
+    pis->iph = TALER_MERCHANT_product_lock2 (
+      TALER_TESTING_interpreter_get_context (is),
+      pis->merchant_url,
+      pis->product_id,
+      pis->uuid,
+      pis->duration,
+      pis->quantity,
+      pis->quantity_frac,
+      true,
+      &lock_product_cb,
+      pis);
+  }
+  else
+  {
+    pis->iph = TALER_MERCHANT_product_lock (
+      TALER_TESTING_interpreter_get_context (is),
+      pis->merchant_url,
+      pis->product_id,
+      pis->uuid,
+      pis->duration,
+      pis->quantity,
+      &lock_product_cb,
+      pis);
+  }
   GNUNET_assert (NULL != pis->iph);
 }
 
@@ -223,6 +251,8 @@ TALER_TESTING_cmd_merchant_lock_product (
                                                    sizeof (uuid));
   pis->duration = duration;
   pis->quantity = quantity;
+  pis->quantity_frac = 0;
+  pis->use_fractional_quantity = false;
 
   {
     struct TALER_TESTING_Command cmd = {
@@ -238,4 +268,31 @@ TALER_TESTING_cmd_merchant_lock_product (
 }
 
 
+struct TALER_TESTING_Command
+TALER_TESTING_cmd_merchant_lock_product2 (
+  const char *label,
+  const char *merchant_url,
+  const char *product_id,
+  struct GNUNET_TIME_Relative duration,
+  uint32_t quantity,
+  uint32_t quantity_frac,
+  bool use_fractional_quantity,
+  unsigned int http_status)
+{
+  struct LockProductState *pis;
+  struct TALER_TESTING_Command cmd;
+
+  cmd = TALER_TESTING_cmd_merchant_lock_product (label,
+                                                 merchant_url,
+                                                 product_id,
+                                                 duration,
+                                                 quantity,
+                                                 http_status);
+  pis = cmd.cls;
+  pis->quantity_frac = quantity_frac;
+  pis->use_fractional_quantity = use_fractional_quantity;
+  return cmd;
+}
+
+
 /* end of testing_api_cmd_lock_product.c */
