@@ -33,8 +33,318 @@
 #include <taler/taler_testing_lib.h>
 #include <taler/taler_signatures.h>
 #include "taler_merchant_service.h"
+#include "taler_merchant_pay_service.h"
 #include "taler_merchant_testing_lib.h"
 
+#ifdef HAVE_DONAU_DONAU_SERVICE_H
+#include <donau/donau_service.h>
+#include <donau/donau_testing_lib.h>
+#include <donau/donau_json_lib.h>
+#endif /* HAVE_DONAU_DONAU_SERVICE_H */
+
+#ifdef HAVE_DONAU_DONAU_SERVICE_H
+/**
+ * Struct for handling the CS approach in signing of the bkps
+ */
+struct CSR_Data
+{
+  /**
+   * Handle to the "batch issue receipt status" operation.
+   */
+  struct DONAU_CsRBatchIssueHandle *csr_handle;
+
+  /**
+   * CS-Nonce
+   */
+  union GNUNET_CRYPTO_BlindSessionNonce nonce;
+
+  /**
+   * batch issue receipt status state
+   */
+  struct StatusState *ss;
+
+  /**
+   * array position in batch issue receipt request (first position is zero)
+   */
+  size_t position;
+};
+
+/**
+ * Handling all data needed for the /pay DONAU CMD.
+ */
+struct MerchantDonauPayData
+{
+  /**
+   * Donau URL.
+   */
+  const char *donau_url;
+
+  /**
+   * Donau keys
+   */
+  struct DONAU_Keys *keys;
+
+  /**
+   * Charity reference
+   */
+  const char *charity_reference;
+
+  /**
+   * Charity id
+   */
+  uint64_t charity_id;
+
+  /**
+   * Amount of the donation
+   */
+  struct TALER_Amount donation_amount;
+
+  /**
+   * Number of BUDIs to create or fetch. Example only.
+   */
+  uint32_t num_bkps;
+
+  /**
+   * Year of the donation
+   */
+  uint64_t year;
+
+  /**
+   * Selected donation unit pub keys for this pay order request
+   */
+  struct DONAU_DonationUnitPublicKey *selected_pks;
+
+  /**
+   * BUDI key pairs used in the payment (blinded_udi + pubkey).
+   */
+  struct DONAU_BlindedUniqueDonorIdentifierKeyPair *bkps;
+
+  /**
+   * Blinding secrets, if needed for each BUDI (CS vs. RSA).
+   */
+  union GNUNET_CRYPTO_BlindingSecretP *blinding_secrets;
+
+  /**
+   * Blinding values. Cs-nonces, cipher.
+   */
+  const struct DONAU_BatchIssueValues **alg_values;
+
+  /**
+   * Hash of the salted donor tax id, if relevant.
+   */
+  struct DONAU_HashDonorTaxId h_donor_tax_id;
+
+  /**
+   * Array of donation receipts;
+   */
+  struct DONAU_DonationReceipt *receipts;
+
+  /**
+   * Array of hashed udis.
+   */
+  struct DONAU_UniqueDonorIdentifierHashP *h_udis;
+
+  /**
+   * If using the CS approach, we might track how many
+   * asynchronous calls are still pending, etc.
+   */
+  unsigned int cs_pending;
+
+  /**
+   * Budis Key Pairs json
+   */
+  json_t *budis_json;
+};
+
+
+/**
+ * Prepares the donau data for the /pay CMD.
+ *
+ * @param is interpreter state
+ * @param ss donau data to prepare
+ */
+static enum GNUNET_GenericReturnValue
+prepare_donau_data (struct TALER_TESTING_Interpreter *is,
+                    struct MerchantDonauPayData *ss)
+{
+  /* Get charity id and the charity private key from trait */
+  {
+    const struct TALER_TESTING_Command *charity_post_cmd;
+    const uint64_t *charity_id;
+
+    charity_post_cmd = TALER_TESTING_interpreter_lookup_command (
+      is,
+      ss->charity_reference);
+
+    if (GNUNET_OK !=
+        TALER_TESTING_get_trait_charity_id (charity_post_cmd,
+                                            &charity_id))
+    {
+      GNUNET_break (0);
+      return GNUNET_SYSERR;
+    }
+    ss->charity_id = (uint64_t) *(charity_id);
+  }
+
+  /* Get donau keys from trait */
+  {
+    const struct TALER_TESTING_Command *keys_cmd;
+    struct DONAU_Keys *keys;
+
+    keys_cmd = TALER_TESTING_interpreter_get_command (is,
+                                                      "donau");
+
+    if (GNUNET_OK !=
+        TALER_TESTING_get_trait_donau_keys (keys_cmd,
+                                            &keys))
+    {
+      GNUNET_break (0);
+      return GNUNET_SYSERR;
+    }
+    ss->keys = keys;
+  }
+
+  /* Get selected_pks + num_bkps*/
+  {
+    enum GNUNET_GenericReturnValue sret;
+
+    sret = DONAU_select_donation_unit_keys_for_amount (
+      ss->keys,
+      &ss->donation_amount,
+      ss->year,
+      &ss->selected_pks,
+      &ss->num_bkps);
+
+    if (GNUNET_SYSERR == sret)
+    {
+      GNUNET_break (0);
+      TALER_TESTING_interpreter_fail (is);
+      return GNUNET_SYSERR;
+    }
+    if ((GNUNET_NO == sret) || (0 == ss->num_bkps))
+    {
+      GNUNET_log (GNUNET_ERROR_TYPE_WARNING,
+                  "Could not compose exact amount from donation units\n");
+      TALER_TESTING_interpreter_fail (is);
+      return GNUNET_NO;
+    }
+  }
+
+  /* Get BUDIsKP */
+  {
+    ss->bkps
+      = GNUNET_new_array (ss->num_bkps,
+                          struct DONAU_BlindedUniqueDonorIdentifierKeyPair);
+    ss->blinding_secrets
+      = GNUNET_new_array (ss->num_bkps,
+                          union GNUNET_CRYPTO_BlindingSecretP);
+    ss->receipts
+      = GNUNET_new_array (ss->num_bkps,
+                          struct DONAU_DonationReceipt);
+    ss->alg_values
+      = GNUNET_new_array (ss->num_bkps,
+                          const struct DONAU_BatchIssueValues *);
+    ss->h_udis
+      = GNUNET_new_array (ss->num_bkps,
+                          struct DONAU_UniqueDonorIdentifierHashP);
+
+    for (size_t cnt = 0; cnt < ss->num_bkps; cnt++)
+    {
+      struct DONAU_UniqueDonorIdentifierNonce *udi_nonce;
+      struct DONAU_BudiMasterSecretP ps;
+      const struct DONAU_BatchIssueValues *alg_values;
+      struct DONAU_BlindedUniqueDonorIdentifier *blinded_udi;
+      struct DONAU_UniqueDonorIdentifierHashP *udi_hash;
+
+      DONAU_donation_unit_pub_hash (&ss->selected_pks[cnt],
+                                    &ss->bkps[cnt].h_donation_unit_pub);
+
+      ss->receipts[cnt].h_donation_unit_pub
+        = ss->bkps[cnt].h_donation_unit_pub;
+      udi_nonce
+        = &ss->receipts[cnt].nonce;
+      blinded_udi
+        = &ss->bkps[cnt].blinded_udi;
+      udi_hash = &ss->h_udis[cnt];
+
+      GNUNET_CRYPTO_random_block (GNUNET_CRYPTO_QUALITY_STRONG,
+                                  &ps,
+                                  sizeof (ps));
+      GNUNET_CRYPTO_random_block (GNUNET_CRYPTO_QUALITY_NONCE,
+                                  udi_nonce,
+                                  sizeof (*udi_nonce));
+      switch (ss->selected_pks[cnt].bsign_pub_key->cipher)
+      {
+      case GNUNET_CRYPTO_BSA_RSA:
+        alg_values = DONAU_donation_unit_ewv_rsa_singleton ();
+        DONAU_budi_secret_create (&ps,
+                                  alg_values,
+                                  &ss->blinding_secrets[cnt]);
+        GNUNET_assert (GNUNET_OK ==
+                       DONAU_donation_unit_blind (
+                         &ss->selected_pks[cnt],
+                         &ss->blinding_secrets[cnt],
+                         NULL,                    /* no cs-nonce needed for rsa */
+                         udi_nonce,
+                         &ss->h_donor_tax_id,
+                         alg_values,
+                         udi_hash,
+                         blinded_udi));
+        ss->alg_values[cnt] = alg_values;
+        break;
+      case GNUNET_CRYPTO_BSA_CS:
+        GNUNET_log (GNUNET_ERROR_TYPE_WARNING,
+                    "CS donation-unit key not yet supported – skip");
+        return GNUNET_NO;
+        /* FIXME: BUG-#### Cs support missing/broken for donau
+        struct CSR_Data *csr_data = GNUNET_new (struct CSR_Data);
+        TALER_cs_withdraw_nonce_derive ( // FIXME: write new method
+          (struct TALER_PlanchetMasterSecretP *) &ps,
+          &csr_data->nonce.cs_nonce);
+        csr_data->ss = is;
+        csr_data->position = cnt;
+
+        csr_data->csr_handle = DONAU_csr_issue (
+          TALER_TESTING_interpreter_get_context (is),
+          TALER_TESTING_get_donau_url (is),
+          &ss->selected_pks[cnt],
+          &csr_data->nonce.cs_nonce,
+          &cs_stage_two_callback,
+          csr_data);
+        if (NULL == csr_data->csr_handle)
+        {
+          GNUNET_break (0);
+        }
+        ss->cs_pending++; */
+        break;
+      default:
+        GNUNET_break (0);
+      }
+    }
+
+    {
+      json_t *budikeypairs = json_array ();
+
+      GNUNET_assert (NULL != budikeypairs);
+      for (size_t i = 0; i < ss->num_bkps; i++)
+      {
+        json_t *budikeypair = GNUNET_JSON_PACK (
+          GNUNET_JSON_pack_data_auto ("h_donation_unit_pub",
+                                      &ss->bkps[i].h_donation_unit_pub),
+          DONAU_JSON_pack_blinded_donation_identifier ("blinded_udi",
+                                                       &ss->bkps[i].blinded_udi)
+          );
+
+        /* steal the reference into the array */
+        GNUNET_assert (0 == json_array_append_new (budikeypairs,
+                                                   budikeypair));
+      }
+      ss->budis_json = budikeypairs;
+    }
+  }
+  return GNUNET_OK;
+};
+#endif /* HAVE_DONAU_DONAU_SERVICE_H */
 
 /**
  * State for a /pay CMD.
@@ -115,6 +425,11 @@ struct PayState
    */
   unsigned int num_issued_tokens;
 
+  /**
+   * Number of donau_tokens in @e issued_tokens.
+   */
+  unsigned int num_donau_tokens;
+
   /**
    * The session for which the payment is made.
    */
@@ -135,6 +450,12 @@ struct PayState
    */
   int choice_index;
 
+#ifdef HAVE_DONAU_DONAU_SERVICE_H
+  /**
+   * Donau data, if required.
+   */
+  struct MerchantDonauPayData donau_data;
+#endif /* HAVE_DONAU_DONAU_SERVICE_H */
 };
 
 
@@ -297,7 +618,7 @@ build_coins (struct TALER_MERCHANT_PayCoin **pc,
       const struct TALER_DenominationSignature *denom_sig;
       const struct TALER_Amount *denom_value;
       const struct TALER_EXCHANGE_DenomPublicKey *denom_pub;
-      const struct TALER_AgeCommitmentHash *h_age_commitment;
+      const struct TALER_AgeCommitmentHashP *h_age_commitment;
 
       GNUNET_assert (GNUNET_OK ==
                      TALER_TESTING_get_trait_coin_priv (coin_cmd,
@@ -461,7 +782,8 @@ pay_cb (void *cls,
   if (MHD_HTTP_OK == pr->hr.http_status)
   {
     ps->merchant_sig = pr->details.ok.merchant_sig;
-    if (ps->num_issued_tokens != pr->details.ok.num_tokens)
+    if (ps->num_issued_tokens + ps->num_donau_tokens !=
+        pr->details.ok.num_tokens)
     {
       GNUNET_log (GNUNET_ERROR_TYPE_ERROR,
                   "Unexpected number of tokens issued. "
@@ -755,23 +1077,15 @@ pay_run (void *cls,
           const char *ierror_name = NULL;
           unsigned int ierror_line = 0;
 
-          struct GNUNET_JSON_Specification ispec[] = {
+          struct GNUNET_JSON_Specification typespec[] = {
             GNUNET_JSON_spec_string ("type",
                                      &kind),
-            GNUNET_JSON_spec_string ("token_family_slug",
-                                     &slug),
-            GNUNET_JSON_spec_uint32 ("key_index",
-                                     &key_index),
-            GNUNET_JSON_spec_mark_optional (
-              GNUNET_JSON_spec_uint32 ("count",
-                                       &count),
-              NULL),
             GNUNET_JSON_spec_end ()
           };
 
           if (GNUNET_OK !=
               GNUNET_JSON_parse (output,
-                                 ispec,
+                                 typespec,
                                  &ierror_name,
                                  &ierror_line))
           {
@@ -784,64 +1098,172 @@ pay_run (void *cls,
             TALER_TESTING_FAIL (is);
           }
 
-          if (0 != strcmp ("token", kind))
+          if (0 == strcmp ("tax-receipt",
+                           kind))
           {
-            continue;
-          }
+            const json_t *donau_urls;
 
-          GNUNET_array_grow (ps->issued_tokens,
-                             ps->num_issued_tokens,
-                             ps->num_issued_tokens + count);
-
-          for (unsigned int k = 0; k < count; k++)
-          {
-            struct TALER_MERCHANT_PrivateTokenDetails *details =
-              &ps->issued_tokens[ps->num_issued_tokens - count + k];
+            // For test, we care only about the presence of it
+            struct GNUNET_JSON_Specification donauspec[] = {
+              GNUNET_JSON_spec_array_const ("donau_urls",
+                                            &donau_urls),
+              GNUNET_JSON_spec_end ()
+            };
 
             if (GNUNET_OK !=
-                find_token_public_key (token_families,
-                                       slug,
-                                       key_index,
-                                       &details->issue_pub))
+                GNUNET_JSON_parse (output,
+                                   donauspec,
+                                   &ierror_name,
+                                   &ierror_line))
             {
+              GNUNET_log (GNUNET_ERROR_TYPE_ERROR,
+                          "Parser failed on %s:%u for input `%s'\n",
+                          ierror_name,
+                          ierror_line,
+                          json_dumps (output,
+                                      JSON_INDENT (2)));
               TALER_TESTING_FAIL (is);
             }
 
-            /* Only RSA is supported for now. */
-            GNUNET_assert (GNUNET_CRYPTO_BSA_RSA ==
-                           details->issue_pub.public_key->cipher);
-
-            TALER_token_blind_input_copy (&details->blinding_inputs,
-                                          TALER_token_blind_input_rsa_singleton ()
-                                          );
-            /* FIXME: Where to get details->blinding_inputs from? */
-            TALER_token_use_setup_random (&details->master);
-            TALER_token_use_setup_priv (&details->master,
-                                        &details->blinding_inputs,
-                                        &details->token_priv);
-            TALER_token_use_blinding_secret_create (&details->master,
-                                                    &details->blinding_inputs,
-                                                    &details->blinding_secret);
-            GNUNET_CRYPTO_eddsa_key_get_public (&details->token_priv.private_key
-                                                ,
-                                                &details->token_pub.public_key);
-            GNUNET_CRYPTO_hash (&details->token_pub.public_key,
-                                sizeof (struct GNUNET_CRYPTO_EcdsaPublicKey),
-                                &details->h_token_pub.hash);
-            details->envelope.blinded_pub = GNUNET_CRYPTO_message_blind_to_sign
-                                            (
-              details->issue_pub.public_key,
-              &details->blinding_secret,
-              NULL, /* FIXME: Add session nonce to support CS tokens */
-              &details->h_token_pub.hash,
-              sizeof (details->h_token_pub.hash),
-              details->blinding_inputs.blinding_inputs);
-
-            if (NULL == details->envelope.blinded_pub)
+#ifdef HAVE_DONAU_DONAU_SERVICE_H
             {
-              GNUNET_break (0);
+              const char   *donau_url_str;
+
+              if ( (NULL == donau_urls) ||
+                   (0 == json_array_size (donau_urls)) )
+              {
+                GNUNET_log (GNUNET_ERROR_TYPE_ERROR,
+                            "No donau_urls found in output\n");
+                TALER_TESTING_FAIL (is);
+              }
+
+              donau_url_str = json_string_value (json_array_get (donau_urls,
+                                                                 0));
+              if (NULL == donau_url_str)
+              {
+                GNUNET_log (GNUNET_ERROR_TYPE_ERROR,
+                            "First entry in donau_urls is not a string\n");
+                TALER_TESTING_FAIL (is);
+              }
+
+              ps->donau_data.donau_url = GNUNET_strdup (donau_url_str);
+
+              if (NULL != ps->donau_data.charity_reference)
+              {
+                switch (prepare_donau_data (is,
+                                            &ps->donau_data))
+                {
+                case GNUNET_OK:
+                  break;
+                case GNUNET_NO:
+                  TALER_TESTING_interpreter_next (ps->is);
+                  return;
+                case GNUNET_SYSERR:
+                  TALER_TESTING_FAIL (is);
+                  return;
+                }
+                ps->num_donau_tokens = ps->donau_data.num_bkps;
+              }
+            }
+#else /* HAVE_DONAU_DONAU_SERVICE_H */
+            /* SIMPLY NOTHING */
+#endif /* HAVE_DONAU_DONAU_SERVICE_H */
+          }
+
+          if (0 == strcmp ("token",
+                           kind))
+          {
+            struct GNUNET_JSON_Specification ispec[] = {
+              GNUNET_JSON_spec_string ("token_family_slug",
+                                       &slug),
+              GNUNET_JSON_spec_uint32 ("key_index",
+                                       &key_index),
+              GNUNET_JSON_spec_mark_optional (
+                GNUNET_JSON_spec_uint32 ("count",
+                                         &count),
+                NULL),
+              GNUNET_JSON_spec_end ()
+            };
+
+            if (GNUNET_OK !=
+                GNUNET_JSON_parse (output,
+                                   ispec,
+                                   &ierror_name,
+                                   &ierror_line))
+            {
+              GNUNET_log (GNUNET_ERROR_TYPE_ERROR,
+                          "Parser failed on %s:%u for input `%s'\n",
+                          ierror_name,
+                          ierror_line,
+                          json_dumps (output,
+                                      JSON_INDENT (2)));
               TALER_TESTING_FAIL (is);
             }
+
+            if (0 != strcmp ("token",
+                             kind))
+            {
+              continue;
+            }
+
+            GNUNET_array_grow (
+              ps->issued_tokens,
+              ps->num_issued_tokens,
+              ps->num_issued_tokens + count + ps->num_donau_tokens);
+
+            for (unsigned int k = 0; k < count; k++)
+            {
+              struct TALER_MERCHANT_PrivateTokenDetails *details =
+                &ps->issued_tokens[ps->num_issued_tokens - count + k
+                                   + ps->num_donau_tokens];
+
+              if (GNUNET_OK !=
+                  find_token_public_key (token_families,
+                                         slug,
+                                         key_index,
+                                         &details->issue_pub))
+              {
+                TALER_TESTING_FAIL (is);
+              }
+
+              /* Only RSA is supported for now. */
+              GNUNET_assert (GNUNET_CRYPTO_BSA_RSA ==
+                             details->issue_pub.public_key->cipher);
+
+              TALER_token_blind_input_copy (&details->blinding_inputs,
+                                            TALER_token_blind_input_rsa_singleton ()
+                                            );
+              /* FIXME: Where to get details->blinding_inputs from? */
+              TALER_token_use_setup_random (&details->master);
+              TALER_token_use_setup_priv (&details->master,
+                                          &details->blinding_inputs,
+                                          &details->token_priv);
+              TALER_token_use_blinding_secret_create (&details->master,
+                                                      &details->blinding_inputs,
+                                                      &details->blinding_secret)
+              ;
+              GNUNET_CRYPTO_eddsa_key_get_public (
+                &details->token_priv.private_key,
+                &details->token_pub.public_key);
+              GNUNET_CRYPTO_hash (&details->token_pub.public_key,
+                                  sizeof (struct GNUNET_CRYPTO_EcdsaPublicKey),
+                                  &details->h_token_pub.hash);
+              details->envelope.blinded_pub =
+                GNUNET_CRYPTO_message_blind_to_sign
+                (
+                  details->issue_pub.public_key,
+                  &details->blinding_secret,
+                  NULL,   /* FIXME: Add session nonce to support CS tokens */
+                  &details->h_token_pub.hash,
+                  sizeof (details->h_token_pub.hash),
+                  details->blinding_inputs.blinding_inputs);
+
+              if (NULL == details->envelope.blinded_pub)
+              {
+                GNUNET_break (0);
+                TALER_TESTING_FAIL (is);
+              }
+            }
           }
         }
       }
@@ -850,8 +1272,6 @@ pay_run (void *cls,
     default:
       TALER_TESTING_FAIL (is);
     }
-
-
   }
 
   {
@@ -901,8 +1321,8 @@ pay_run (void *cls,
                      ps->num_issued_tokens);
   for (unsigned int i = 0; i<len_output_tokens; i++)
   {
-    output_tokens[i].envelope.blinded_pub = ps->issued_tokens[i].envelope.
-                                            blinded_pub;
+    output_tokens[i].envelope.blinded_pub
+      = ps->issued_tokens[i].envelope.blinded_pub;
   }
 
   if (GNUNET_OK !=
@@ -915,34 +1335,80 @@ pay_run (void *cls,
                                                 &h_proposal))
     TALER_TESTING_FAIL (is);
   ps->h_contract_terms = *h_proposal;
-  ps->oph = TALER_MERCHANT_order_pay (
-    TALER_TESTING_interpreter_get_context (is),
-    ps->merchant_url,
-    ps->session_id,
-    h_proposal,
-    ps->choice_index,
-    &ps->total_amount,
-    &max_fee,
-    &merchant_pub,
-    merchant_sig,
-    timestamp,
-    refund_deadline,
-    pay_deadline,
-    &h_wire,
-    order_id,
-    npay_coins,
-    pay_coins,
-    len_use_tokens,
-    use_tokens,
-    len_output_tokens,
-    output_tokens,
-    &pay_cb,
-    ps);
+
+  /* New logic of setting pay params */
+  {
+    struct GNUNET_CURL_Context *ctx =
+      TALER_TESTING_interpreter_get_context (is);
+    struct TALER_MERCHANT_OrderPayOption opts[32];
+    size_t oi = 0;
+
+    ps->oph = TALER_MERCHANT_order_pay_create (ctx,
+                                               &pay_cb,
+                                               ps);
+
+    if (NULL == ps->oph)
+      TALER_TESTING_FAIL (is);
+
+#define ADD(_opt)  opts[oi++] = (_opt)
+    ADD (TALER_MERCHANT_ORDER_PAY_OPTION_MERCHANT_URL (ps->merchant_url));
+    ADD (TALER_MERCHANT_ORDER_PAY_OPTION_ORDER_ID      (order_id));
+    if (NULL != ps->session_id)
+      ADD (TALER_MERCHANT_ORDER_PAY_OPTION_SESSION_ID  (ps->session_id));
+    ADD (TALER_MERCHANT_ORDER_PAY_OPTION_H_CONTRACT    (h_proposal));
+    if (ps->choice_index >= 0)
+      ADD (TALER_MERCHANT_ORDER_PAY_OPTION_CHOICE_INDEX (ps->choice_index));
+    ADD (TALER_MERCHANT_ORDER_PAY_OPTION_AMOUNT        (&ps->total_amount));
+    ADD (TALER_MERCHANT_ORDER_PAY_OPTION_MAX_FEE       (&max_fee));
+    ADD (TALER_MERCHANT_ORDER_PAY_OPTION_MERCHANT_PUB  (&merchant_pub));
+    ADD (TALER_MERCHANT_ORDER_PAY_OPTION_TIMESTAMP     (timestamp));
+    ADD (TALER_MERCHANT_ORDER_PAY_OPTION_REFUND_DEADLINE (refund_deadline));
+    ADD (TALER_MERCHANT_ORDER_PAY_OPTION_PAY_DEADLINE  (pay_deadline));
+    ADD (TALER_MERCHANT_ORDER_PAY_OPTION_H_WIRE        (&h_wire));
+    ADD (TALER_MERCHANT_ORDER_PAY_OPTION_COINS         (npay_coins,
+                                                        pay_coins));
+    if (len_use_tokens > 0)
+      ADD (TALER_MERCHANT_ORDER_PAY_OPTION_INPUT_TOKENS (len_use_tokens,
+                                                         use_tokens));
+    if (len_output_tokens > 0)
+      ADD (TALER_MERCHANT_ORDER_PAY_OPTION_OUTPUT_TOKENS (len_output_tokens,
+                                                          output_tokens));
+
+#ifdef HAVE_DONAU_DONAU_SERVICE_H
+    if (ps->donau_data.charity_reference)
+    {
+      ADD (
+        TALER_MERCHANT_ORDER_PAY_OPTION_DONAU_URL  (ps->donau_data.donau_url));
+      ADD (TALER_MERCHANT_ORDER_PAY_OPTION_DONAU_YEAR (ps->donau_data.year));
+      ADD (TALER_MERCHANT_ORDER_PAY_OPTION_DONAU_BUDIS (
+             ps->donau_data.budis_json));
+    }
+#endif
+    ADD (TALER_MERCHANT_ORDER_PAY_OPTION_TERMINATE ());
+#undef ADD
+
+    if (TALER_MERCHANT_OPOEC_OK !=
+        TALER_MERCHANT_order_pay_set_options (ps->oph,
+                                              opts,
+                                              oi))
+      TALER_TESTING_FAIL (is);
+
+    if (TALER_MERCHANT_OPOEC_OK !=
+        TALER_MERCHANT_order_pay_start (ps->oph))
+      TALER_TESTING_FAIL (is);
+  }
+
   GNUNET_array_grow (pay_coins,
                      npay_coins,
                      0);
-  if (NULL == ps->oph)
-    TALER_TESTING_FAIL (is);
+
+  GNUNET_array_grow (use_tokens,
+                     len_use_tokens,
+                     0);
+
+  GNUNET_array_grow (output_tokens,
+                     len_output_tokens,
+                     0);
 }
 
 
@@ -964,7 +1430,7 @@ pay_cleanup (void *cls,
                 "Command `%s' did not complete.\n",
                 TALER_TESTING_interpreter_get_current_label (
                   ps->is));
-    TALER_MERCHANT_order_pay_cancel (ps->oph);
+    TALER_MERCHANT_order_pay_cancel1 (ps->oph);
   }
 
   GNUNET_free (ps);
@@ -1062,16 +1528,17 @@ pay_traits (void *cls,
 
 
 struct TALER_TESTING_Command
-TALER_TESTING_cmd_merchant_pay_order_choices (const char *label,
-                                              const char *merchant_url,
-                                              unsigned int http_status,
-                                              const char *proposal_reference,
-                                              const char *coin_reference,
-                                              const char *amount_with_fee,
-                                              const char *amount_without_fee,
-                                              const char *session_id,
-                                              int choice_index,
-                                              const char *token_reference)
+TALER_TESTING_cmd_merchant_pay_order_choices (
+  const char *label,
+  const char *merchant_url,
+  unsigned int http_status,
+  const char *proposal_reference,
+  const char *coin_reference,
+  const char *amount_with_fee,
+  const char *amount_without_fee,
+  const char *session_id,
+  int choice_index,
+  const char *token_reference)
 {
   struct PayState *ps;
 
@@ -1099,26 +1566,94 @@ TALER_TESTING_cmd_merchant_pay_order_choices (const char *label,
 }
 
 
+#ifdef HAVE_DONAU_DONAU_SERVICE_H
+
 struct TALER_TESTING_Command
-TALER_TESTING_cmd_merchant_pay_order (const char *label,
-                                      const char *merchant_url,
-                                      unsigned int http_status,
-                                      const char *proposal_reference,
-                                      const char *coin_reference,
-                                      const char *amount_with_fee,
-                                      const char *amount_without_fee,
-                                      const char *session_id)
+TALER_TESTING_cmd_merchant_pay_order_donau (
+  const char *label,
+  const char *merchant_url,
+  unsigned int http_status,
+  const char *proposal_reference,
+  const char *coin_reference,
+  const char *amount_with_fee,
+  const char *amount_without_fee,
+  const char *amount_donation,
+  const char *session_id,
+  int choice_index,
+  const char *charity_reference,
+  uint64_t year,
+  const char *donor_tax_id,
+  const char *salt)
 {
-  return TALER_TESTING_cmd_merchant_pay_order_choices (label,
-                                                       merchant_url,
-                                                       http_status,
-                                                       proposal_reference,
-                                                       coin_reference,
-                                                       amount_with_fee,
-                                                       amount_without_fee,
-                                                       session_id,
-                                                       -1,
-                                                       NULL);
+  struct PayState *ps;
+
+  ps = GNUNET_new (struct PayState);
+  ps->http_status = http_status;
+  ps->proposal_reference = proposal_reference;
+  ps->coin_reference = coin_reference;
+  ps->merchant_url = merchant_url;
+  ps->amount_with_fee = amount_with_fee;
+  ps->amount_without_fee = amount_without_fee;
+  ps->session_id = session_id;
+  ps->token_reference = NULL;
+  ps->choice_index = choice_index;
+  ps->donau_data.year = year;
+  ps->donau_data.num_bkps = 5;
+  ps->donau_data.charity_reference = charity_reference;
+  if (GNUNET_OK !=
+      TALER_string_to_amount (amount_donation,
+                              &ps->donau_data.donation_amount))
+  {
+    GNUNET_assert (0);
+  }
+
+  /* Compute h_donor_tax_id directly into ps->donau_data: */
+  if (! DONAU_compute_salted_tax_id_hash (donor_tax_id,
+                                          salt,
+                                          ps->donau_data.h_donor_tax_id.hash))
+  {
+    GNUNET_assert (0);
+  }
+
+  {
+    struct TALER_TESTING_Command cmd = {
+      .cls = ps,
+      .label = label,
+      .run = &pay_run,
+      .cleanup = &pay_cleanup,
+      .traits = &pay_traits
+    };
+
+    return cmd;
+  }
+}
+
+
+#endif /* HAVE_DONAU_DONAU_SERVICE_H */
+
+
+struct TALER_TESTING_Command
+TALER_TESTING_cmd_merchant_pay_order (
+  const char *label,
+  const char *merchant_url,
+  unsigned int http_status,
+  const char *proposal_reference,
+  const char *coin_reference,
+  const char *amount_with_fee,
+  const char *amount_without_fee,
+  const char *session_id)
+{
+  return TALER_TESTING_cmd_merchant_pay_order_choices (
+    label,
+    merchant_url,
+    http_status,
+    proposal_reference,
+    coin_reference,
+    amount_with_fee,
+    amount_without_fee,
+    session_id,
+    -1,
+    NULL);
 }
 
 
