@@ -28,6 +28,9 @@
 #include <gnunet/gnunet_time_lib.h>
 #include <jansson.h>
 #include <stdint.h>
+#include "taler_merchant_util.h"
+#include <stdlib.h>
+#include <math.h>
 #include <taler/taler_exchange_service.h>
 #include <taler/taler_testing_lib.h>
 #include "taler_merchant_service.h"
@@ -490,6 +493,7 @@ orders_run2 (void *cls,
   {
     char *ctok;
     struct TALER_MERCHANT_InventoryProduct pd;
+    double quantity_double = 0.0;
 
     /* Token syntax is "[product_id]/[quantity]" */
     ctok = strchr (token, '/');
@@ -497,17 +501,59 @@ orders_run2 (void *cls,
     {
       *ctok = '\0';
       ctok++;
-      if (1 != sscanf (ctok,
-                       "%u",
-                       &pd.quantity))
       {
-        GNUNET_break (0);
-        break;
+        char *endptr;
+
+        quantity_double = strtod (ctok,
+                                  &endptr);
+        if ( (endptr == ctok) || ('\0' != *endptr) ||
+             (! isfinite (quantity_double)) || (quantity_double < 0.0))
+        {
+          GNUNET_break (0);
+          break;
+        }
       }
     }
     else
     {
-      pd.quantity = 1;
+      quantity_double = 1.0;
+    }
+    if (quantity_double <= 0.0)
+    {
+      GNUNET_break (0);
+      break;
+    }
+
+    {
+      double quantity_floor;
+      double frac;
+      uint64_t quantity_int;
+      uint32_t quantity_frac_local = 0;
+      long long scaled;
+
+      quantity_floor = floor (quantity_double);
+      frac = quantity_double - quantity_floor;
+      quantity_int = (uint64_t) quantity_floor;
+      if (frac < 0.0)
+      {
+        GNUNET_break (0);
+        break;
+      }
+      scaled = llround (frac * (double) MERCHANT_UNIT_FRAC_BASE);
+      if (scaled < 0)
+      {
+        GNUNET_break (0);
+        break;
+      }
+      if (scaled >= (long long) MERCHANT_UNIT_FRAC_BASE)
+      {
+        quantity_int++;
+        scaled -= MERCHANT_UNIT_FRAC_BASE;
+      }
+      quantity_frac_local = (uint32_t) scaled;
+      pd.quantity = quantity_int;
+      pd.quantity_frac = quantity_frac_local;
+      pd.use_fractional_quantity = (0 != quantity_frac_local);
     }
     pd.product_id = token;
 
